<?php


/***************************************************************************//**
 * Construit et renvoie l'Html des données d'un mot, données passées dans
 * un Array associatif
 * 
 * @param Array $r Array associatif des données du mot, issues de la base sqlite
 * @param Object $db Objet PDO d'accès à la base sqlite du dico concerné
 * @param string $fdic Nom du fichier du dictionnaire courant
 * @param Array $lstrel Array associatif des libellés des relations 
 * @param string $phpself Url des liens de relations (terminé par '?' ou '&amp;')
 * @param string $dom Domaine de recherche dans le dico (entrées, notices...)
 * @param string $stxt Motif recherché (pour le surlignage et pour les liens directs)
 * @param boolean $srl Surlignage du mot recherché
 * @param string $lg Langue d'interface 
 * @param string $htmplug Chaîne des liens de plugins du dico
 * @return string Le code html de la notice du mot à afficher 
 */
function getHtmOfWordData($r, $db, $lstrel, $fdic, $phpself, $dom, $stxt, $srl, $lg, $htmplug)
    {
    
    global $sqlng_str;  // Array des chaînes de langue
    
    $htm = '';          // html de la notice du mot (sera retourné par la fonction)
    
    
    // -------------------------------------------------------------
    // --- Mot d'entrée (précédé des liens de plugins éventuels)    
    // -------------------------------------------------------------
    
    $t = $r['word'];
    
    if ($srl && $dom == 'entr') // surlignage optionnel (seulement la 1ère occurrence)
        {
        $t = preg_replace('/' . str_replace('%', '.*?', $stxt) . '/i', '<span class="sl1">\\0</span>', $t, 1);
        }
    
    // Mise en gras qu'on arrète au premier crochet ouvrant
    // (les crochets encadrent les codes grammaticaux généralement)
    $t = '<b>' . $t;
    if(strpos($t, '['))
        {
        $t = str_replace('[', '</b>[', $t);
        }
    else
        {
        $t .= '</b>';
        }
    
    // On adapte la chaîne des plugins au mot concerné
    $htmplug = str_replace('[!!!wd!!!]', urlencode($r['word']), $htmplug);
    
    
    // On décale la notice (tout ce qui suit le mot d'entrée)
    $htm .= "\n" . '<p class="wd">' . $t . $htmplug . '</p>
    <blockquote>';
    
    
    // -------------------
    // --- Phonétique     
    // -------------------
    
    $t = trim($r['phon']);
    
    if (strlen($t))
        {
        $htm .= "\n" . '<p><i>' . $sqlng_str[21] . '</i> ' . $t . '</p>';
        }
    
    
    // ---------------------------
    // --- Traduction courte      
    // ---------------------------
    
    $t = trim($r['short']);
    
    if ($srl && $dom == 'not') // surlignage optionnel (toutes les occurences)
        {
        $t = preg_replace('/' . str_replace('%', '.*?', $stxt) . '/i', '<span class="sl2">\\0</span>', $t);
        }
    
    $t = nl2br($t); // conversion sauts de lignes > <br />
    
    $htm .= "\n" . '<p>' . $t . '</p>';
    
    
    
    
    // --------------------------------
    // --- Infos, traduction longue    
    // --------------------------------
    
    $t = trim($r['long']);
    
    if (strlen($t))
        {
        if ($srl && $dom == 'not') // surlignage optionnel (toutes les occurences)
            {
            $t = preg_replace('/' . str_replace('%', '.*?', $stxt) . '/i', '<span class="sl2">\\0</span>', $t);
            }
        
        $t = nl2br($t); // conversion sauts de lignes > <br />
        
        $htm .= "\n" . '<p>' . $t . '</p>';
        }
    
    
    // ---------------------------------------------
    // --- Conversion des balises xdxf de couleur   
    // ---------------------------------------------
    
    $htm = preg_replace('/<c c="#(.*?)">(.*?)<\/c>/i', '<span style="color:#\\1">\\2</span>', $htm);
    
    
    // -------------------------------------
    // --- Liens internes de type {{mot}}   
    // -------------------------------------
    
    $htm = preg_replace('/\{\{(.*?)\}\}/i',
           '<a href="' . $phpself . 'sqlng_fdic=' . $fdic . '&amp;sqlng_stxt=\\1&amp;sqlng_dom=' . $dom . '&amp;sqlng_lg=' . $lg . '">\\1</a>',
           $htm);
    
    
    
    // -----------------------------------------------------
    // --- Blocs des relations (racines, synonymes, etc.)   
    // -----------------------------------------------------
    
    foreach ($lstrel as $krel=>$ttrrel)
        {
        $t = trim($r[$krel]); // contenu du champ de la relation
        if (strlen($t))
            {
            $htm .= "\n" . '<p class="ttrrel">' . $ttrrel . '</p><ul>';
            $cibles = explode(';', $t);
            foreach($cibles as $cible)
                {
                $cible = trim($cible);
                $rel = $db->query('SELECT word FROM Data WHERE wordid = ' . $db->quote($cible));   
                $rel = $rel->fetch(PDO::FETCH_ASSOC);
                $htm .= "\n" . '<li><a href="' . $phpself . 'sqlng_fdic=' . $fdic . '&amp;sqlng_wid=' . $cible . '&amp;sqlng_dom=' . $dom . '&amp;sqlng_lg=' . $lg . '">' .  $rel['word'] . '</a></li>';
                }
            $htm .= '</ul>';
            }
        }    
    
    
    
       
    // -------------------------------------------------------------------------
    // Sécurité avant l'affichage : 'htmlspecialchars' ne peut être utilisé car 
    // le texte issu de la base sqlite peut contenir des balise html de format  
    // (gras, couleur, etc.) MAIS l'utilisateur peut mettre ce qu'il veut dans  
    // cette base "dictionnaire" et certains dicos peuvent être d'origine       
    // douteuse... On nettoie donc les principaux trucs susceptibles d'entraîner
    // des problèmes de XSS pour le site-hôte                                   
    // -------------------------------------------------------------------------
    
    $htm = str_ireplace('<script', '', $htm);
    $htm = str_ireplace('<noscript', '', $htm);
    $htm = str_ireplace('<meta', '', $htm);
    $htm = str_ireplace('document.location', '', $htm);
    $htm = str_ireplace('<?php', '', $htm);
    
    $htm = str_ireplace('src=', '', $htm);
    $htm = str_ireplace('!src!', 'src=', $htm); // rétablissement des protections
    
    return $htm . "\n" . '</blockquote>';
    
    }

?>