<?php

/***************************************************************************//**
 *
 * DECLINATOR pour SqLinguae
 * =========================
 *
 * Declinator est un greffon pour le script SqLinguae (visualiseur de 
 * dictionnaires en ligne) mais peut également être utilisé indépendamment de
 * celui-ci en tant que classe PHP de déclinaison/conjugaison latine, à la 
 * condition que le mot latin fourni en paramètre suive les régles de codage
 * grammatical exposées dans le fichier 'manuel_fr.htm'.
 *   
 */

class cl_LatinWord
    {


    // ========================================
    // === Propriétés publiques de la classe   
    // ========================================
        
    // flags de statut
    var $isUnknownStatus = False;         //(Boolean) expression à statut grammatical inconnu
    var $toNotDecl = False;                //(Boolean) expression à ne pas décliner même si potentiellement déclinable(locutions par exemple)
    var $isDeclUnknown = False;            //(Boolean) expression potentiellement déclinable mais non déclinée car déclinaison non connue
    var $isDeclNone = False;               //(Boolean) expression réellement indéclinable
    var $isDeclined = False;               //(Boolean) expression déclinable et a été effectivement déclinée (des données sont dans Decl() )
    var $isConjugated = False;             //(Boolean) expression qui a été effectivement conjuguée (des données sont dans conj() )
        
    // flags grammaticaux
    var $isSubstantive = False;            //(Boolean) substantif
    var $isAdjective = False;              //(Boolean) adjectif
    var $isVerb = False;                   //(Boolean) verbe
    var $isAdverb = False;                 //(Boolean) adverbe
    var $isRelative = False;               //(Boolean) pronom-adjectif relatif
    var $isNumeral = False;                //(Boolean) nombre (cardinal ou ordinal)
    var $isPrefix = False;                 //(Boolean) préfixe
    var $isSuffix = False;                 //(Boolean) suffixe
    var $isLocution = False;               //(Boolean) locution
    var $isConjunction = False;            //(Boolean) conjonction
    var $isDemonstrative = False;          //(Boolean) démonstratif
    var $isAbbreviation = False;           //(Boolean) abréviation
    var $isMasculine = False;              //(Boolean) masculin
    var $isFeminine = False;               //(Boolean) féminin
    var $isNeutral = False;                //(Boolean) neutre
    var $isSingular = False;               //(Boolean) singulier
    var $isPlural = False;                 //(Boolean) pluriel
    var $isTransitive = False;             //(Boolean) transitif
    var $isIntransitive = False;           //(Boolean) intransitif
    var $isPreposition = False;            //(Boolean) préposition
    var $isRem = False;                    //(Boolean) commentaire
    var $isWithAccusative = False;         //(Boolean) ... + accusatif
    var $isWithAblative = False;           //(Boolean) ... + ablatif
    var $isSymbol = False;                 //(Boolean) symbole
    var $isComparative = False;            //(Boolean) comparatif
    var $isSuperlative = False;            //(Boolean) superlatif
    var $isFromLatin = False;              //(Boolean) provenant du Latin
    var $isFromGreek = False;              //(Boolean) provenant du Grec
    var $isDecl1 = False;                  //(Boolean) 1ère déclinaison
    var $isDecl2 = False;                  //(Boolean) 2ème déclinaison
    var $isDecl3 = False;                  //(Boolean) 3ème déclinaison
    var $isDecl4 = False;                  //(Boolean) 4ème déclinaison
    var $isDecl5 = False;                  //(Boolean) 5ème déclinaison
    var $isGroupA = False;                 //(Boolean) groupe A (adjectifs)
    var $isGroupB = False;                 //(Boolean) groupe B (adjectifs)
    var $isGroupC = False;                 //(Boolean) groupe C (adjectifs)
    var $isConj1 = False;                  //(Boolean) 1ère conjugaison
    var $isConj2 = False;                  //(Boolean) 2ème conjugaison
    var $isConj3 = False;                  //(Boolean) 3ème conjugaison
    var $isConj4 = False;                  //(Boolean) 4ème conjugaison
    var $isConjM = False;                  //(Boolean) conjugaison mixte
    var $isConjSum = False;                //(Boolean) conjugaison de Sum et ses dérivés
    var $isConjFero = False;               //(Boolean) conjugaison de Fero et ses dérivés
        
        
    var $GramInfo = '';                    //(String) descriptif convivial du code grammatical
        
    var $Conj = Array();                   // Séquence de la conjugaison : liste de (racine, désinence)
    var $Decl = Array();                   // Séquence de la déclinaison : liste de (racine, désinence)
        
    // Données des verbes
    var $Vinfinitif = '';                  //(String) --- infinitif
    var $Vradic1 = '';                     //(String) --- radical du présent
    var $Vradic2 = '';                     //(String) --- radical du parfait
    var $Vradic3 = '';                     //(String) --- radical du supin
    
    
    // ========================================
    // === Propriétés privées de la classe     
    // ========================================
        
        
    // --- Mot de base (mot nu, nettoyé de la suite de désinences, codes grammaticaux etc.)
    //   ex : "Rosa/ae [s f 1]" > MotBase = "Rosa"
    var $__MotBase = '';                         //(String)
        
        
    // ---- Données des verbes ---
    var $curNumConj = '';                       //(String) --- n° de conjugaison courante
        
        
    // =======(fin des initialisations des propriétés)=======




/***************************************************************************//**
 * Constructeur de l'instance.
 *   
 * Décline ou conjugue 'item'
 *
 * @param string $item Entrée complète de dictionnaire : mot + code grammatical
 */   
function cl_LatinWord($item)      
    {
    $this->__definirDeclinaison($item);
    }   




/***************************************************************************//**
 * Tente d'attribuer des données grammaticales à un mot qui en est       
 * explicitement dépourvu.                                               
 *                                                                       
 * Si succès de l'analyse, un code grammatical est attribué et la        
 * procédure de définition de la déclinaison es automatiquement relancée.
 *                                                                       
 * @param string $item Mot brut complet                                  
 *                                                                       
 * @return none                                                          
 */
    function AnalyseAutomatique($item)
        {
        
        $idGram = '';
        $sp = '';      // séparateur des désinences du mot source
        
        // ------------
        
        
        foreach (Array('/', ',', ', ', ' ') as $sp)
            
            // === numéraux ===
                
            if (strpos($item, 'unus' . $sp . 'a' . $sp . 'um') !== False)
                {
                $idGram = 'num1';
                }
            elseif ((strpos($item, 'duo' . $sp . 'duae' . $sp . 'duo') !== False) or (strpos($item, 'duo' . $sp . 'duæ' . $sp . 'duo') !== False))
                {
                $idGram = 'num2';
                }
            elseif (strpos($item, 'tres' . $sp . 'tres' . $sp . 'tria') !== False)
                {
                $idGram = 'num3';
                }
            
            // === Adjectifs ===
                
            elseif (strpos($item, 'us' . $sp . 'a' . $sp . 'um') !== False)
                {
                $idGram = 'adj A';
                }
            elseif (strpos($item, 'er' . $sp . 'era' . $sp . 'erum') !== False)
                {
                $idGram = 'adj Aa';
                }
            elseif (strpos($item, 'is' . $sp . 'is' . $sp . 'e') !== False)
                {
                $idGram = 'adj Bp';
                }
            elseif (strpos($item, 'ex' . $sp . 'icis') !== False)
                {
                $idGram = 'adj Bex';
                }
            elseif ((strpos($item, 'er' . $sp . 'cris' . $sp . 'cre') !== False) or (strpos($item, 'er' . $sp . 'tris' . $sp . 'tre') !== False))
                {
                $idGram = 'adj Ber';
                }
            elseif ((strpos($item, 'ax' . $sp . 'acis') !== False) or (strpos($item, 'ox' . $sp . 'ocis') !== False))
                {
                $idGram = 'adj Bx';
                }
            elseif (strpos($item, 'or' . $sp . 'oris') !== False)
                {
                $idGram = 'adj Bor';
                }
            elseif (strpos($item, 'us' . $sp . 'eris') !== False)
                {
                $idGram = 'adj Bus';
                }
            elseif (strpos($item, 'oides' . $sp . 'oidis') !== False)
                {
                $idGram = 'adj Ci';
                }
            elseif (strpos($item, 'odes' . $sp . 'odis') !== False)
                {
                $idGram = 'adj Co';
                }
            elseif (strpos($item, 'ior' . $sp . 'ior' . $sp . 'ius') !== False)
                {
                $idGram = 'cmp';
                }
            
            // === Substantifs ===
                
            elseif ((strpos($item, 'a' . $sp . 'ae') !== False) or (strpos($item, 'a' . $sp . 'æ') !== False))
                {
                $idGram = 's 1';
                }
            elseif (strpos($item, 'us' . $sp . 'i') !== False)
                {
                $idGram = 's 2';
                }
            elseif (strpos($item, 'um' . $sp . 'i') !== False)
                {
                $idGram = 's n 2';
                }
            elseif (strpos($item, 'us' . $sp . 'us') !== False)
                {
                $idGram = 's 4';
                }
            elseif (strpos($item, 'us' . $sp . 'ua') !== False)
                {
                $idGram = 's n 4';
                }
            elseif (strpos($item, 'es' . $sp . 'ei') !== False)
                {
                $idGram = 's 5';
                }
            
            // === Verbes ===
                
            elseif (strpos($item, $sp . 'are' . $sp) !== False)
                {
                $idGram = 'v1';
                }
            elseif ((strpos($item, $sp . 'ire' . $sp) !== False) and ($sp == '/'))
                {
                $idGram = 'v4';
                }
            elseif ((strpos($item, $sp . 'ere' . $sp) !== False) and ($sp == '/'))
                {
                if (strpos($item, 'eo' . $sp) !== False)
                    {   
                    $idGram = 'v2';
                    }
                elseif (strpos($item, 'io' . $sp) !== False)
                    {
                    $idGram = 'vm';
                    }
                elseif (strpos($item, 'o' . $sp) !== False)
                    {
                    $idGram = 'v3';
                    }
                }
                
                
            elseif (strpos($item, 'esse' . $sp) !== False)
                {
                $idGram = 'vs';
                }
            elseif ((strpos($item, 'ferre' . $sp) !== False) and ($sp == '/'))
                {
                $idGram = 'vf';
                }
                
            // ---(end if)
            
                
            if ($idGram != '')
                {
                break;
                }
        
        // ---(fin for $sp)
        
        
        // ================
        
        if ($idGram != '')
            {
            $item = $item . ' [' . $idGram . ']';
            $this->__definirDeclinaison($item);
            }
            
        }






/***************************************************************************//**
 * Remplit la liste publique de conjugaison, Conj(), pour un temps et un mode
 * donnés à partir des racines et du numéro de conjugaison (antérieurement
 * établis)
 *  
        
           Paramètres :
                           mode    = mode de la conjugaison
                           temps   = temps de la conjugaison
        
        Procédure utilisée par le constructeur puis ensuite utilisable comme méthode publique
         pour mettre à jour la liste Conj() de l'objet avec d'autres temps/modes
         que l'indicatif présent
        
        Cartographie des index de la liste :
                   0-5  : voix active
                   6-11 : voix passive

 *     
 */
    function ConjUpdate($mode, $temps)
        {
    
      
        
        $n = 0;
        $flagParfaitDefectif = False;
        $flagSupinDefectif = False;
        // -----------------------
        
        // réinitialiser la liste (0-11)
        $this->Conj = Array();
        
        for ($n = 0; $n <= 11; $n++)
            {
            $this->Conj[$n] = new MotCas();
            }
        
            
        // homogénéiser la casse des paramètres (> minuscules)
        $mode = strToLower($mode);
        $temps = strToLower($temps);
        
        $this->isConjugated = True;
        
        // ==============================================
        // === Remplissage des racines du verbe conjugué 
        // ==============================================
        
        if (in_array($temps, Array('present', 'imparfait', 'futur')))
            {
            for ($n = 0; $n <= 11; $n++)
                {
                $this->Conj[$n]->Rac = $this->Vradic1;         // racine du présent
                }
            }
        else
            {
            if ($this->Vradic2 != '-')
                {
                for ($n = 0; $n <= 5; $n++)                      // actif
                    {
                    $this->Conj[$n]->Rac = $this->Vradic2 ;    // racine du parfait
                    }
                }
            else
                {
                $flagParfaitDefectif = True;
                }
            
            if ($this->Vradic3 != '-')
                {
                for ($n = 6; $n <= 11; $n++)                    // passif
                    {
                    $this->Conj[$n]->Rac = $this->Vradic3;     // racine du supin
                    }
                }
            else
                {
                $flagSupinDefectif = True;
                }
            }
            
            
        // ========================================================
        // === VOIX ACTIVE ========================================
        // ========================================================
        
        if ($mode == 'indicatif')                         // actif/indicatif
            {
            if ($temps == 'present')                      // actif/indicatif/présent
                {
                if ($this->curNumConj == '1')
                    {
                    $this->Conj[0]->Des = 'o';
                    $this->Conj[1]->Des = 'as';
                    $this->Conj[2]->Des = 'at';
                    $this->Conj[3]->Des = 'amus';
                    $this->Conj[4]->Des = 'atis';
                    $this->Conj[5]->Des = 'ant';
                    }
                elseif ($this->curNumConj == '2')
                    {
                    $this->Conj[0]->Des = 'eo';
                    $this->Conj[1]->Des = 'es';
                    $this->Conj[2]->Des = 'et';
                    $this->Conj[3]->Des = 'emus';
                    $this->Conj[4]->Des = 'etis';
                    $this->Conj[5]->Des = 'ent';
                    }
                elseif ($this->curNumConj == '3')
                    {
                    $this->Conj[0]->Des = 'o';
                    $this->Conj[1]->Des = 'is';
                    $this->Conj[2]->Des = 'it';
                    $this->Conj[3]->Des = 'imus';
                    $this->Conj[4]->Des = 'itis';
                    $this->Conj[5]->Des = 'unt';
                    }
                elseif ($this->curNumConj == '4')
                    {
                    $this->Conj[0]->Des = 'io';
                    $this->Conj[1]->Des = 'is';
                    $this->Conj[2]->Des = 'it';
                    $this->Conj[3]->Des = 'imus';
                    $this->Conj[4]->Des = 'itis';
                    $this->Conj[5]->Des = 'iunt';
                    }
                elseif ($this->curNumConj == 'm')
                    {
                    $this->Conj[0]->Des = 'io';
                    $this->Conj[1]->Des = 'is';
                    $this->Conj[2]->Des = 'it';
                    $this->Conj[3]->Des = 'imus';
                    $this->Conj[4]->Des = 'itis';
                    $this->Conj[5]->Des = 'iunt';
                    }
                elseif ($this->curNumConj == 's')  // sum et dérivés
                    {
                    $this->Conj[0]->Des = 'sum';
                    $this->Conj[1]->Des = 'es';
                    $this->Conj[2]->Des = 'est';
                    $this->Conj[3]->Des = 'sumus';
                    $this->Conj[4]->Des = 'estis';
                    $this->Conj[5]->Des = 'sunt';
                    }
                elseif ($this->curNumConj == 'f')  // fero et dérivés
                    {
                    $this->Conj[0]->Des = 'o';
                    $this->Conj[1]->Des = 's';
                    $this->Conj[2]->Des = 't';
                    $this->Conj[3]->Des = 'imus';
                    $this->Conj[4]->Des = 'tis';
                    $this->Conj[5]->Des = 'unt';
                    }
                }
            elseif ($temps == 'imparfait')                // actif/indicatif/imparfait
                {
                if ($this->curNumConj == '1')
                    {
                    $this->Conj[0]->Des = 'abam';
                    $this->Conj[1]->Des = 'abas';
                    $this->Conj[2]->Des = 'abat';
                    $this->Conj[3]->Des = 'abamus';
                    $this->Conj[4]->Des = 'abatis';
                    $this->Conj[5]->Des = 'abant';
                    }
                elseif (in_array($this->curNumConj, Array('2', '3', 'f')))
                    {
                    $this->Conj[0]->Des = 'ebam';
                    $this->Conj[1]->Des = 'ebas';
                    $this->Conj[2]->Des = 'ebat';
                    $this->Conj[3]->Des = 'ebamus';
                    $this->Conj[4]->Des = 'ebatis';
                    $this->Conj[5]->Des = 'ebant';
                    }
                elseif (in_array($this->curNumConj, Array('4', 'm')))
                    {
                    $this->Conj[0]->Des = 'iebam';
                    $this->Conj[1]->Des = 'iebas';
                    $this->Conj[2]->Des = 'iebat';
                    $this->Conj[3]->Des = 'iebamus';
                    $this->Conj[4]->Des = 'iebatis';
                    $this->Conj[5]->Des = 'iebant';
                    }
                elseif ($this->curNumConj ==  'm')
                    {
                    $this->Conj[0]->Des = 'iam';
                    $this->Conj[1]->Des = 'ies';
                    $this->Conj[2]->Des = 'iet';
                    $this->Conj[3]->Des = 'iemus';
                    $this->Conj[4]->Des = 'ietis';
                    $this->Conj[5]->Des = 'ient';
                    }
                elseif ($this->curNumConj == 's')
                    {
                    $this->Conj[0]->Des = 'eram';
                    $this->Conj[1]->Des = 'eras';
                    $this->Conj[2]->Des = 'erat';
                    $this->Conj[3]->Des = 'eramus';
                    $this->Conj[4]->Des = 'eratis';
                    $this->Conj[5]->Des = 'erant';
                    }
                }
            elseif ($temps == 'futur')                    // actif/indicatif/futur
                {
                if ($this->curNumConj == '1')
                    {
                    $this->Conj[0]->Des = 'abo';
                    $this->Conj[1]->Des = 'abis';
                    $this->Conj[2]->Des = 'abit';
                    $this->Conj[3]->Des = 'abimus';
                    $this->Conj[4]->Des = 'abitis';
                    $this->Conj[5]->Des = 'abunt';
                    }
                elseif ($this->curNumConj == '2')
                    {
                    $this->Conj[0]->Des = 'ebo';
                    $this->Conj[1]->Des = 'ebis';
                    $this->Conj[2]->Des = 'ebit';
                    $this->Conj[3]->Des = 'ebimus';
                    $this->Conj[4]->Des = 'ebitis';
                    $this->Conj[5]->Des = 'ebunt';
                    }
                elseif (in_array($this->curNumConj, Array('3', 'f')))
                    {
                    $this->Conj[0]->Des = 'am';
                    $this->Conj[1]->Des = 'es';
                    $this->Conj[2]->Des = 'et';
                    $this->Conj[3]->Des = 'emus';
                    $this->Conj[4]->Des = 'etis';
                    $this->Conj[5]->Des = 'ent';
                    }
                elseif (in_array($this->curNumConj, Array('4', 'm')))
                    {
                    $this->Conj[0]->Des = 'iam';
                    $this->Conj[1]->Des = 'ies';
                    $this->Conj[2]->Des = 'iet';
                    $this->Conj[3]->Des = 'iemus';
                    $this->Conj[4]->Des = 'ietis';
                    $this->Conj[5]->Des = 'ient';
                    }
                elseif ($this->curNumConj == 's')
                    {
                    $this->Conj[0]->Des = 'ero';
                    $this->Conj[1]->Des = 'eris';
                    $this->Conj[2]->Des = 'erit';
                    $this->Conj[3]->Des = 'erimus';
                    $this->Conj[4]->Des = 'eritis';
                    $this->Conj[5]->Des = 'erunt';
                    }
                }
            elseif ($temps == 'parfait')                  // actif/indicatif/parfait
                {
                if ($flagParfaitDefectif == False)
                    {
                    // toutes conjugaisons
                    $this->Conj[0]->Des = 'i';
                    $this->Conj[1]->Des = 'isti';
                    $this->Conj[2]->Des = 'it';
                    $this->Conj[3]->Des = 'imus';
                    $this->Conj[4]->Des = 'istis';
                    $this->Conj[5]->Des = 'erunt/ere';
                    }
                }
            elseif ($temps == 'plus-que-parfait')         // actif/indicatif/Plus-que-parfait
                {
                if ($flagParfaitDefectif == False)
                    {
                    //toutes conjugaisons
                    $this->Conj[0]->Des = 'eram';
                    $this->Conj[1]->Des = 'eras';
                    $this->Conj[2]->Des = 'erat';
                    $this->Conj[3]->Des = 'eramus';
                    $this->Conj[4]->Des = 'eratis';
                    $this->Conj[5]->Des = 'erant';
                    }
                }
            elseif ($temps == 'futur anterieur')           // actif/indicatif/futur antérieur
                {
                if ($flagParfaitDefectif == False)
                    {
                    // toutes conjugaisons
                    $this->Conj[0]->Des = 'ero';
                    $this->Conj[1]->Des = 'eris';
                    $this->Conj[2]->Des = 'erit';
                    $this->Conj[3]->Des = 'erimus';
                    $this->Conj[4]->Des = 'eritis';
                    $this->Conj[5]->Des = 'erint';
                    }
                }
            }
        elseif ($mode == 'subjonctif/conditionnel')     // actif/subjonctif
            {
            if ($temps == 'present')                    // actif/subjonctif/présent
                {
                if ($this->curNumConj == '1')
                    {
                    $this->Conj[0]->Des = 'em';
                    $this->Conj[1]->Des = 'es';
                    $this->Conj[2]->Des = 'et';
                    $this->Conj[3]->Des = 'emus';
                    $this->Conj[4]->Des = 'etis';
                    $this->Conj[5]->Des = 'ent';
                    }
                elseif ($this->curNumConj == '2')
                    {
                    $this->Conj[0]->Des = 'eam';
                    $this->Conj[1]->Des = 'eas';
                    $this->Conj[2]->Des = 'eat';
                    $this->Conj[3]->Des = 'eamus';
                    $this->Conj[4]->Des = 'eatis';
                    $this->Conj[5]->Des = 'eant';
                    }
                elseif (in_array($this->curNumConj, Array('3', 'f')))
                    {
                    $this->Conj[0]->Des = 'am';
                    $this->Conj[1]->Des = 'as';
                    $this->Conj[2]->Des = 'at';
                    $this->Conj[3]->Des = 'amus';
                    $this->Conj[4]->Des = 'atis';
                    $this->Conj[5]->Des = 'ant';
                    }
                elseif (in_array($this->curNumConj, Array('4', 'm')))
                    {
                    $this->Conj[0]->Des = 'iam';
                    $this->Conj[1]->Des = 'ias';
                    $this->Conj[2]->Des = 'iat';
                    $this->Conj[3]->Des = 'iamus';
                    $this->Conj[4]->Des = 'iatis';
                    $this->Conj[5]->Des = 'iant';
                    }
                elseif ($this->curNumConj == 's')
                    {
                    $this->Conj[0]->Des = 'sim';
                    $this->Conj[1]->Des = 'sis';
                    $this->Conj[2]->Des = 'sit';
                    $this->Conj[3]->Des = 'simus';
                    $this->Conj[4]->Des = 'sitis';
                    $this->Conj[5]->Des = 'sint';
                    }
                }
            elseif ($temps == 'imparfait')                // actif/subjonctif/imparfait
                {
                if ($this->curNumConj == '1')
                    {
                    $this->Conj[0]->Des = 'arem';
                    $this->Conj[1]->Des = 'ares';
                    $this->Conj[2]->Des = 'aret';
                    $this->Conj[3]->Des = 'aremus';
                    $this->Conj[4]->Des = 'aretis';
                    $this->Conj[5]->Des = 'arent';
                    }
                elseif (in_array($this->curNumConj, Array('2', '3', 'm')))
                    {
                    $this->Conj[0]->Des = 'erem';
                    $this->Conj[1]->Des = 'eres';
                    $this->Conj[2]->Des = 'eret';
                    $this->Conj[3]->Des = 'eremus';
                    $this->Conj[4]->Des = 'eretis';
                    $this->Conj[5]->Des = 'erent';
                    }
                elseif ($this->curNumConj == '4')
                    {
                    $this->Conj[0]->Des = 'irem';
                    $this->Conj[1]->Des = 'ires';
                    $this->Conj[2]->Des = 'iret';
                    $this->Conj[3]->Des = 'iremus';
                    $this->Conj[4]->Des = 'iretis';
                    $this->Conj[5]->Des = 'irent';
                    }
                elseif ($this->curNumConj == 's')
                    {
                    $this->Conj[0]->Des = 'essem/(forem)';
                    $this->Conj[1]->Des = 'esses/(fores)';
                    $this->Conj[2]->Des = 'esset/(foret)';
                    $this->Conj[3]->Des = 'essemus';
                    $this->Conj[4]->Des = 'essetis';
                    $this->Conj[5]->Des = 'essent/(forent)';
                    }
                elseif ($this->curNumConj == 'f')
                    {
                    $this->Conj[0]->Des = 'errem';
                    $this->Conj[1]->Des = 'erres';
                    $this->Conj[2]->Des = 'erret';
                    $this->Conj[3]->Des = 'erremus';
                    $this->Conj[4]->Des = 'erretis';
                    $this->Conj[5]->Des = 'errent';
                    }
                }
            elseif ($temps == 'parfait')                  // actif/subjonctif/parfait
                {
                if ($flagParfaitDefectif == False)
                    {
                    // toutes conjugaisons
                    $this->Conj[0]->Des = 'erim';
                    $this->Conj[1]->Des = 'eris';
                    $this->Conj[2]->Des = 'erit';
                    $this->Conj[3]->Des = 'erimus';
                    $this->Conj[4]->Des = 'eritis';
                    $this->Conj[5]->Des = 'erint';
                    }
                }
            elseif ($temps == 'plus-que-parfait')         // actif/subjonctif/plus-que-parfait
                {
                if ($flagParfaitDefectif == False)
                    {
                    // toutes conjugaisons
                    $this->Conj[0]->Des = 'issem';
                    $this->Conj[1]->Des = 'isses';
                    $this->Conj[2]->Des = 'isset';
                    $this->Conj[3]->Des = 'issemus';
                    $this->Conj[4]->Des = 'issetis';
                    $this->Conj[5]->Des = 'issent';
                    }
                }
            }    
        
        // ====================================================
        // === VOIX PASSIVE ===================================
        // ====================================================
        
        
        if ($this->curNumConj == 's')                  // pas de passif pour 'sum'
            {
            for ($n = 6; $n <= 11; $n++)  // (n = 6 To 11)    
                {
                $this->Conj[$n]->Rac = '';
                $this->Conj[$n]->Des = '';
                }
            return ;
            }
        
        // --------------------------------------
        
        elseif ($mode == 'indicatif')                   // passif/indicatif
            {
            if ($temps == 'present')                  // passif/indicatif/présent
                {
                if ($this->curNumConj == '1')
                    {
                    $this->Conj[6]->Des = 'or';
                    $this->Conj[7]->Des = 'aris/are';
                    $this->Conj[8]->Des = 'atur';
                    $this->Conj[9]->Des = 'amur';
                    $this->Conj[10]->Des = 'amini';
                    $this->Conj[11]->Des = 'antur';
                    }
                elseif ($this->curNumConj == '2')
                    {
                    $this->Conj[6]->Des = 'eor';
                    $this->Conj[7]->Des = 'eris/ere';
                    $this->Conj[8]->Des = 'etur';
                    $this->Conj[9]->Des = 'emur';
                    $this->Conj[10]->Des = 'emini';
                    $this->Conj[11]->Des = 'entur';
                    }
                elseif ($this->curNumConj == '3')
                    {
                    $this->Conj[6]->Des = 'or';
                    $this->Conj[7]->Des = 'eris/ere';
                    $this->Conj[8]->Des = 'itur';
                    $this->Conj[9]->Des = 'imur';
                    $this->Conj[10]->Des = 'imini';
                    $this->Conj[11]->Des = 'untur';
                    }
                elseif ($this->curNumConj == '4')
                    {
                    $this->Conj[6]->Des = 'ior';
                    $this->Conj[7]->Des = 'iris/ire';
                    $this->Conj[8]->Des = 'itur';
                    $this->Conj[9]->Des = 'imur';
                    $this->Conj[10]->Des = 'imini';
                    $this->Conj[11]->Des = 'iuntur';
                    }
                elseif ($this->curNumConj == 'm')
                    {
                    $this->Conj[6]->Des = 'ior';
                    $this->Conj[7]->Des = 'eris/ere';
                    $this->Conj[8]->Des = 'itur';
                    $this->Conj[9]->Des = 'imur';
                    $this->Conj[10]->Des = 'imini';
                    $this->Conj[11]->Des = 'iuntur';
                    }
                elseif ($this->curNumConj == 'f')
                    {
                    $this->Conj[6]->Des = 'or';
                    $this->Conj[7]->Des = 'ris';
                    $this->Conj[8]->Des = 'tur';
                    $this->Conj[9]->Des = 'imur';
                    $this->Conj[10]->Des = 'imini';
                    $this->Conj[11]->Des = 'untur';
                    }
                }
            elseif ($temps == 'imparfait')                // passif/indicatif/imparfait
                {
                if ($this->curNumConj == '1')
                    {
                    $this->Conj[6]->Des = 'abar';
                    $this->Conj[7]->Des = 'abaris/abare';
                    $this->Conj[8]->Des = 'abatur';
                    $this->Conj[9]->Des = 'abamur';
                    $this->Conj[10]->Des = 'abamini';
                    $this->Conj[11]->Des = 'abantur';
                    }
                elseif (in_array($this->curNumConj, Array('2', '3', 'f')))
                    {
                    $this->Conj[6]->Des = 'ebar';
                    $this->Conj[7]->Des = 'ebaris/ebare';
                    $this->Conj[8]->Des = 'ebatur';
                    $this->Conj[9]->Des = 'ebamur';
                    $this->Conj[10]->Des = 'ebamini';
                    $this->Conj[11]->Des = 'ebantur';
                    }
                elseif (in_array($this->curNumConj, Array('4', 'm')))
                    {
                    $this->Conj[6]->Des = 'iebar';
                    $this->Conj[7]->Des = 'iebaris/iebare';
                    $this->Conj[8]->Des = 'iebatur';
                    $this->Conj[9]->Des = 'iebamur';
                    $this->Conj[10]->Des = 'iebamini';
                    $this->Conj[11]->Des = 'iebantur';
                    }
                }
            elseif ($temps == 'futur')                    // passif/indicatif/futur
                {
                if ($this->curNumConj == '1')
                    {
                    $this->Conj[6]->Des = 'abor';
                    $this->Conj[7]->Des = 'aberis/abere';
                    $this->Conj[8]->Des = 'abitur';
                    $this->Conj[9]->Des = 'abimur';
                    $this->Conj[10]->Des = 'abimini';
                    $this->Conj[11]->Des = 'abuntur';
                    }
                elseif ($this->curNumConj == '2')
                    {
                    $this->Conj[6]->Des = 'ebor';
                    $this->Conj[7]->Des = 'eberis/ebere';
                    $this->Conj[8]->Des = 'ebitur';
                    $this->Conj[9]->Des = 'ebimur';
                    $this->Conj[10]->Des = 'ebimini';
                    $this->Conj[11]->Des = 'ebuntur';
                    }
                elseif (in_array($this->curNumConj, Array('3', 'f')))
                    {
                    $this->Conj[6]->Des = 'ar';
                    $this->Conj[7]->Des = 'eris/ere';
                    $this->Conj[8]->Des = 'etur';
                    $this->Conj[9]->Des = 'emur';
                    $this->Conj[10]->Des = 'emini';
                    $this->Conj[11]->Des = 'entur';
                    }
                elseif (in_array($this->curNumConj, Array('4', 'm')))
                    {
                    $this->Conj[6]->Des = 'iar';
                    $this->Conj[7]->Des = 'ieris/iere';
                    $this->Conj[8]->Des = 'ietur';
                    $this->Conj[9]->Des = 'iemur';
                    $this->Conj[10]->Des = 'iemini';
                    $this->Conj[11]->Des = 'ientur';
                    }
                }
            elseif ($temps == 'parfait')                  // passif/indicatif/parfait
                {
                if ($flagSupinDefectif == False)
                    {
                    if ($this->curNumConj != 's') // toutes autres conjugaisons que sum
                        {
                        $this->Conj[6]->Des = 'us/a/um sum';
                        $this->Conj[7]->Des = 'us/a/um es';
                        $this->Conj[8]->Des = 'us/a/um est';
                        $this->Conj[9]->Des = 'i/æ/a sumus';
                        $this->Conj[10]->Des = 'i/æ/a estis';
                        $this->Conj[11]->Des = 'i/æ/a sunt';
                        }
                    }
                }
            elseif ($temps == 'plus-que-parfait')         // passif/indicatif/Plus-que-parfait
                {
                if ($flagSupinDefectif == False)
                    {
                    if ($this->curNumConj != 's')         // toutes autres conjugaisons que sum
                        {
                        $this->Conj[6]->Des = 'us/a/um eram';
                        $this->Conj[7]->Des = 'us/a/um eras';
                        $this->Conj[8]->Des = 'us/a/um erat';
                        $this->Conj[9]->Des = 'i/æ/a eramus';
                        $this->Conj[10]->Des = 'i/æ/a eratis';
                        $this->Conj[11]->Des = 'i/æ/a erant';
                        }
                    }
                }
            elseif ($temps == 'futur anterieur')            // passif/indicatif/futur antérieur
                {
                if ($flagSupinDefectif == False)
                    {
                    if ($this->curNumConj != 's')          // toutes autres conjugaisons que sum
                        {
                        $this->Conj[6]->Des = 'us/a/um ero';
                        $this->Conj[7]->Des = 'us/a/um eris';
                        $this->Conj[8]->Des = 'us/a/um erit';
                        $this->Conj[9]->Des = 'i/æ/a erimus';
                        $this->Conj[10]->Des = 'i/æ/a eritis';
                        $this->Conj[11]->Des = 'i/æ/a erunt';
                        }
                    }
                }
                
            }
        elseif ($mode == 'subjonctif/conditionnel')         // passif/subjonctif
            {
            if ($temps == 'present')                      // passif/subjonctif/présent
                {
                if ($this->curNumConj == '1')
                    {
                    $this->Conj[6]->Des = 'er';
                    $this->Conj[7]->Des = 'eris/ere';
                    $this->Conj[8]->Des = 'etur';
                    $this->Conj[9]->Des = 'emur';
                    $this->Conj[10]->Des = 'emini';
                    $this->Conj[11]->Des = 'entur';
                    }
                elseif ($this->curNumConj == '2')
                    {
                    $this->Conj[6]->Des = 'ear';
                    $this->Conj[7]->Des = 'earis/eare';
                    $this->Conj[8]->Des = 'eatur';
                    $this->Conj[9]->Des = 'eamur';
                    $this->Conj[10]->Des = 'eamini';
                    $this->Conj[11]->Des = 'eantur';
                    }
                elseif (in_array($this->curNumConj, Array('3', 'f')))
                    {
                    $this->Conj[6]->Des = 'ar';
                    $this->Conj[7]->Des = 'aris/eare';
                    $this->Conj[8]->Des = 'atur';
                    $this->Conj[9]->Des = 'amur';
                    $this->Conj[10]->Des = 'amini';
                    $this->Conj[11]->Des = 'antur';
                    }
                elseif (in_array($this->curNumConj, Array('4', 'm')))
                    {
                    $this->Conj[6]->Des = 'iar';
                    $this->Conj[7]->Des = 'iaris/iare';
                    $this->Conj[8]->Des = 'iatur';
                    $this->Conj[9]->Des = 'iamur';
                    $this->Conj[10]->Des = 'iamini';
                    $this->Conj[11]->Des = 'iantur';
                    }
                }
            elseif ($temps == 'imparfait')                // passif/subjonctif/imparfait
                {
                if ($this->curNumConj == '1')
                    {
                    $this->Conj[6]->Des = 'arer';
                    $this->Conj[7]->Des = 'areris/arere';
                    $this->Conj[8]->Des = 'aretur';
                    $this->Conj[9]->Des = 'aremur';
                    $this->Conj[10]->Des = 'aremini';
                    $this->Conj[11]->Des = 'arentur';
                    }
                elseif (in_array($this->curNumConj, Array('2', '3', 'm')))
                    {
                    $this->Conj[6]->Des = 'erer';
                    $this->Conj[7]->Des = 'ereris/erere';
                    $this->Conj[8]->Des = 'eretur';
                    $this->Conj[9]->Des = 'eremur';
                    $this->Conj[10]->Des = 'eremini';
                    $this->Conj[11]->Des = 'erentur';
                    }
                elseif ($this->curNumConj == '4')
                    {
                    $this->Conj[6]->Des = 'irer';
                    $this->Conj[7]->Des = 'ireris/irere';
                    $this->Conj[8]->Des = 'iretur';
                    $this->Conj[9]->Des = 'iremur';
                    $this->Conj[10]->Des = 'iremini';
                    $this->Conj[11]->Des = 'irentur';
                    }
                elseif ($this->curNumConj == 'f')
                    {
                    $this->Conj[6]->Des = 'rer';
                    $this->Conj[7]->Des = 'reris';
                    $this->Conj[8]->Des = 'retur';
                    $this->Conj[9]->Des = 'remur';
                    $this->Conj[10]->Des = 'remini';
                    $this->Conj[11]->Des = 'rentur';
                    }
                }
            elseif ($temps == 'parfait')                   // passif/subjonctif/parfait
                {
                if ($flagSupinDefectif == False)
                    {
                    // toutes conjugaisons
                    $this->Conj[6]->Des = 'us/a/um sim';
                    $this->Conj[7]->Des = 'us/a/um sis';
                    $this->Conj[8]->Des = 'us/a/um sit';
                    $this->Conj[9]->Des = 'i/æ/a simus';
                    $this->Conj[10]->Des = 'i/æ/a sitis';
                    $this->Conj[11]->Des = 'i/æ/a sint';
                    }
                }
            elseif ($temps == 'plus-que-parfait')         // passif/subjonctif/plus-que-parfait
                {
                if ($flagSupinDefectif == False)
                    {
                    // toutes conjugaisons
                    $this->Conj[6]->Des = 'us/a/um essem';
                    $this->Conj[7]->Des = 'us/a/um esses';
                    $this->Conj[8]->Des = 'us/a/um esset';
                    $this->Conj[9]->Des = 'i/æ/a essemus';
                    $this->Conj[10]->Des = 'i/æ/a essetis';
                    $this->Conj[11]->Des = 'i/æ/a essent';
                    }
                }
            }
            
            
        return ;

    
        }




/***************************************************************************//**
 */
    function DeclinerAdjectif($Radic, $d)
        {
        
        if ($d == '')
            {
            return;
            }
        
        $i = 0;
        $n = 0;
        // -----
        
        
        $this->isMasculine = True;
        $this->isFeminine = True;
        $this->isNeutral = True;
        
        $this->isDeclined = True;
        
        
        // =============================================================
        if ($this->isSingular)         // === SINGULIER                    
        // =============================================================
            {
            
            // ---------------------------------------
            // --- Masculin singulier                 
            // ---------------------------------------
            
            $i = 0; // index de départ dans Decl
            
            for ($n = 0; $n <= 5; $n++)  // (0 To 5) racine par défaut (= radical)
                {
                $this->Decl[$i + $n]->Rac = $Radic;
                }
                
            if ($d == 'A')
                {
                $this->Decl[$i + 0]->Des = 'us';    // Nominatif
                $this->Decl[$i + 1]->Des = 'e';     // Vocatif
                $this->Decl[$i + 2]->Des = 'um';    // Accusatif
                $this->Decl[$i + 3]->Des = 'i';     // Génitif
                $this->Decl[$i + 4]->Des = 'o';     // Datif
                $this->Decl[$i + 5]->Des = 'o';     // Ablatif
                }
            elseif ($d == 'Aa')
                {
                $this->Decl[$i + 0]->Des = '';      // Nominatif
                $this->Decl[$i + 1]->Des = '';      // Vocatif
                $this->Decl[$i + 2]->Des = 'um';    // Accusatif
                $this->Decl[$i + 3]->Des = 'i';     // Génitif
                $this->Decl[$i + 4]->Des = 'o';     // Datif
                $this->Decl[$i + 5]->Des = 'o';     // Ablatif
                }
            elseif ($d == 'Ab')
                {
                $this->Decl[$i + 0]->Rac = substr($Radic, 0, -1) . 'er';// Nominatif
                $this->Decl[$i + 1]->Rac = substr($Radic, 0, -1) . 'er';// Vocatif
                
                $this->Decl[$i + 0]->Des = '';      // Nominatif
                $this->Decl[$i + 1]->Des = '';      // Vocatif
                $this->Decl[$i + 2]->Des = 'um';    // Accusatif
                $this->Decl[$i + 3]->Des = 'i';     // Génitif
                $this->Decl[$i + 4]->Des = 'o';     // Datif
                $this->Decl[$i + 5]->Des = 'o';     // Ablatif
                }
            elseif ($d == 'Bp')
                {
                $this->Decl[$i + 0]->Des = 'is';    // Nominatif
                $this->Decl[$i + 1]->Des = 'is';    // Vocatif
                $this->Decl[$i + 2]->Des = 'em';    // Accusatif
                $this->Decl[$i + 3]->Des = 'is';    // Génitif
                $this->Decl[$i + 4]->Des = 'i';     // Datif
                $this->Decl[$i + 5]->Des = 'i';     // Ablatif
                }
            elseif ($d == 'Bns')
                {
                $this->Decl[$i + 0]->Rac = substr($Radic, 0, -1); // Nominatif
                $this->Decl[$i + 1]->Rac = substr($Radic, 0, -1); // Vocatif
                
                $this->Decl[$i + 0]->Des = 's';     // Nominatif
                $this->Decl[$i + 1]->Des = 's';     // Vocatif
                $this->Decl[$i + 2]->Des = 'em';    // Accusatif
                $this->Decl[$i + 3]->Des = 'is';    // Génitif
                $this->Decl[$i + 4]->Des = 'i';     // Datif
                $this->Decl[$i + 5]->Des = 'i/e';   // Ablatif
                }
            elseif ($d == 'Bex')
                {
                $this->Decl[$i + 0]->Rac = substr($Radic, 0, -2); // Nominatif
                $this->Decl[$i + 1]->Rac = substr($Radic, 0, -2); // Vocatif
                
                $this->Decl[$i + 0]->Des = 'ex';    // Nominatif
                $this->Decl[$i + 1]->Des = 'ex';    // Vocatif
                $this->Decl[$i + 2]->Des = 'em';    // Accusatif
                $this->Decl[$i + 3]->Des = 'is';    // Génitif
                $this->Decl[$i + 4]->Des = 'i';     // Datif
                $this->Decl[$i + 5]->Des = 'i';     // Ablatif
                }
            elseif ($d == 'Ber')
                {
                $this->Decl[$i + 0]->Rac = substr($Radic, 0, -1); // Nominatif
                $this->Decl[$i + 1]->Rac = substr($Radic, 0, -1); // Vocatif
                
                $this->Decl[$i + 0]->Des = 'er';    // Nominatif
                $this->Decl[$i + 1]->Des = 'er';    // Vocatif
                $this->Decl[$i + 2]->Des = 'em';    // Accusatif
                $this->Decl[$i + 3]->Des = 'is';    // Génitif
                $this->Decl[$i + 4]->Des = 'i';     // Datif
                $this->Decl[$i + 5]->Des = 'i';     // Ablatif
                }
            elseif ($d == 'Bx')
                {
                $this->Decl[$i + 0]->Rac = substr($Radic, 0, -1); // Nominatif
                $this->Decl[$i + 1]->Rac = substr($Radic, 0, -1); // Vocatif
                
                $this->Decl[$i + 0]->Des = 'x';     // Nominatif
                $this->Decl[$i + 1]->Des = 'x';     // Vocatif
                $this->Decl[$i + 2]->Des = 'em';    // Accusatif
                $this->Decl[$i + 3]->Des = 'is';    // Génitif
                $this->Decl[$i + 4]->Des = 'i';     // Datif
                $this->Decl[$i + 5]->Des = 'i';     // Ablatif
                }
            elseif ($d == 'Bor')
                {
                $this->Decl[$i + 0]->Des = '';      // Nominatif
                $this->Decl[$i + 1]->Des = '';      // Vocatif
                $this->Decl[$i + 2]->Des = 'em';    // Accusatif
                $this->Decl[$i + 3]->Des = 'is';    // Génitif
                $this->Decl[$i + 4]->Des = 'i';     // Datif
                $this->Decl[$i + 5]->Des = 'i';     // Ablatif
                }
            elseif ($d == 'Bus')
                {
                $this->Decl[$i + 0]->Rac = substr($Radic, 0, -2); // Nominatif
                $this->Decl[$i + 1]->Rac = substr($Radic, 0, -2); // Vocatif
                
                $this->Decl[$i + 0]->Des = 'us';    // Nominatif
                $this->Decl[$i + 1]->Des = 'us';    // Vocatif
                $this->Decl[$i + 2]->Des = 'em';    // Accusatif
                $this->Decl[$i + 3]->Des = 'is';    // Génitif
                $this->Decl[$i + 4]->Des = 'i';     // Datif
                $this->Decl[$i + 5]->Des = 'e';     // Ablatif
                }
            elseif (in_array($d, Array('Ci', 'Co')))
                {
                $this->Decl[$i + 0]->Des = 'es';    // Nominatif
                $this->Decl[$i + 1]->Des = 'es';    // Vocatif
                $this->Decl[$i + 2]->Des = 'em';    // Accusatif
                $this->Decl[$i + 3]->Des = 'is';    // Génitif
                $this->Decl[$i + 4]->Des = 'i';     // Datif
                $this->Decl[$i + 5]->Des = 'e';     // Ablatif
                }
            elseif ($d == 'cmp')
                {
                $this->Decl[$i + 0]->Des = 'or';    // Nominatif
                $this->Decl[$i + 1]->Des = 'or';    // Vocatif
                $this->Decl[$i + 2]->Des = 'orem';  // Accusatif
                $this->Decl[$i + 3]->Des = 'oris';  // Génitif
                $this->Decl[$i + 4]->Des = 'ori';   // Datif
                $this->Decl[$i + 5]->Des = 'ore';   // Ablatif
                }
            elseif ($d == 'cmp-i') //(hors contexte, il ne semble y avoir que des pluriels : 'plures/a')
                {
                //pass ;
                }
            
            
            
            // -----------------------------------------------------
            // --- Féminin singulier                                
            // -----------------------------------------------------
            
            $i = 12; // index de départ dans Decl
            
            
            for ($n = 0; $n <= 5; $n++)    // (0 To 5) racine par défaut (= radical)
                {
                $this->Decl[$i + $n]->Rac = $Radic;
                }
            
            if (in_array($d, Array('A', 'Aa', 'Ab')))
                {
                $this->Decl[$i + 0]->Des = 'a';     // Nominatif
                $this->Decl[$i + 1]->Des = 'a';     // Vocatif
                $this->Decl[$i + 2]->Des = 'am';    // Accusatif
                $this->Decl[$i + 3]->Des = 'æ';    // Génitif
                $this->Decl[$i + 4]->Des = 'æ';    // Datif
                $this->Decl[$i + 5]->Des = 'a';     // Ablatif
                }
            elseif (in_array($d, Array('Bp', 'Ber')))
                {
                $this->Decl[$i + 0]->Des = 'is';    // Nominatif
                $this->Decl[$i + 1]->Des = 'is';    // Vocatif
                $this->Decl[$i + 2]->Des = 'em';    // Accusatif
                $this->Decl[$i + 3]->Des = 'is';    // Génitif
                $this->Decl[$i + 4]->Des = 'i';     // Datif
                $this->Decl[$i + 5]->Des = 'i';     // Ablatif
                }
            elseif ($d == 'Bns')
                {
                $this->Decl[$i + 0]->Rac = substr($Radic, 0, -1); // Nominatif
                $this->Decl[$i + 1]->Rac = substr($Radic, 0, -1); // Vocatif
                
                $this->Decl[$i + 0]->Des = 's';     // Nominatif
                $this->Decl[$i + 1]->Des = 's';     // Vocatif
                $this->Decl[$i + 2]->Des = 'em';    // Accusatif
                $this->Decl[$i + 3]->Des = 'is';    // Génitif
                $this->Decl[$i + 4]->Des = 'i';     // Datif
                $this->Decl[$i + 5]->Des = 'i/e';   // Ablatif
                }
            elseif ($d == 'Bex')
                {
                $this->Decl[$i + 0]->Rac = substr($Radic, 0, -2);  // Nominatif
                $this->Decl[$i + 1]->Rac = substr($Radic, 0, -2);  // Vocatif
                
                $this->Decl[$i + 0]->Des = 'ex';    // Nominatif
                $this->Decl[$i + 1]->Des = 'ex';    // Vocatif
                $this->Decl[$i + 2]->Des = 'em';    // Accusatif
                $this->Decl[$i + 3]->Des = 'is';    // Génitif
                $this->Decl[$i + 4]->Des = 'i';     // Datif
                $this->Decl[$i + 5]->Des = 'i';     // Ablatif
                }
            elseif ($d == 'Bx')
                {
                $this->Decl[$i + 0]->Rac = substr($Radic, 0, -1); // Nominatif
                $this->Decl[$i + 1]->Rac = substr($Radic, 0, -1); // Vocatif
                
                $this->Decl[$i + 0]->Des = 'x';     // Nominatif
                $this->Decl[$i + 1]->Des = 'x';     // Vocatif
                $this->Decl[$i + 2]->Des = 'em';    // Accusatif
                $this->Decl[$i + 3]->Des = 'is';    // Génitif
                $this->Decl[$i + 4]->Des = 'i';     // Datif
                $this->Decl[$i + 5]->Des = 'i';     // Ablatif
                }
            elseif ($d == 'Bor')
                {
                $this->Decl[$i + 0]->Des = '';      // Nominatif
                $this->Decl[$i + 1]->Des = '';      // Vocatif
                $this->Decl[$i + 2]->Des = 'em';    // Accusatif
                $this->Decl[$i + 3]->Des = 'is';    // Génitif
                $this->Decl[$i + 4]->Des = 'i';     // Datif
                $this->Decl[$i + 5]->Des = 'i';     // Ablatif
                }
            elseif ($d == 'Bus')
                {
                $this->Decl[$i + 0]->Rac = substr($Radic, 0, -2); // Nominatif
                $this->Decl[$i + 1]->Rac = substr($Radic, 0, -2); // Vocatif
                
                $this->Decl[$i + 0]->Des = 'us';    // Nominatif
                $this->Decl[$i + 1]->Des = 'us';    // Vocatif
                $this->Decl[$i + 2]->Des = 'em';    // Accusatif
                $this->Decl[$i + 3]->Des = 'is';    // Génitif
                $this->Decl[$i + 4]->Des = 'i';     // Datif
                $this->Decl[$i + 5]->Des = 'e';     // Ablatif
                }
            elseif (in_array($d, Array('Ci', 'Co')))
                {
                $this->Decl[$i + 0]->Des = 'es';    // Nominatif
                $this->Decl[$i + 1]->Des = 'es';    // Vocatif
                $this->Decl[$i + 2]->Des = 'em';    // Accusatif
                $this->Decl[$i + 3]->Des = 'is';    // Génitif
                $this->Decl[$i + 4]->Des = 'i';     // Datif
                $this->Decl[$i + 5]->Des = 'e';     // Ablatif
                }
            elseif ($d == 'cmp')
                {
                $this->Decl[$i + 0]->Des = 'or';    // Nominatif
                $this->Decl[$i + 1]->Des = 'or';    // Vocatif
                $this->Decl[$i + 2]->Des = 'orem';  // Accusatif
                $this->Decl[$i + 3]->Des = 'oris';  // Génitif
                $this->Decl[$i + 4]->Des = 'ori';   // Datif
                $this->Decl[$i + 5]->Des = 'ore';   // Ablatif
                }
            elseif ($d == 'cmp-i') //(hors contexte, il ne semble y avoir que des pluriels : 'plures/a')
                {
                //pass;
                }
            
            
            // -----------------------------------------------------
            // --- Neutre singulier                                 
            // -----------------------------------------------------
            
            $i = 24; // index de départ dans Decl
            
            
            for ($n = 0; $n <= 5; $n++)        // (0 To 5) racine par défaut (= radical)
                {
                $this->Decl[$i + $n]->Rac = $Radic;
                }
            
            
            if (in_array($d, Array('A', 'Aa', 'Ab')))
                {
                $this->Decl[$i + 0]->Des = 'um';    // Nominatif
                $this->Decl[$i + 1]->Des = 'um';    // Vocatif
                $this->Decl[$i + 2]->Des = 'um';    // Accusatif
                $this->Decl[$i + 3]->Des = 'i';     // Génitif
                $this->Decl[$i + 4]->Des = 'o';     // Datif
                $this->Decl[$i + 5]->Des = 'o';     // Ablatif
                }
            elseif (in_array($d, Array('Bp', 'Ber')))
                {
                $this->Decl[$i + 0]->Des = 'e';     // Nominatif
                $this->Decl[$i + 1]->Des = 'e';     // Vocatif
                $this->Decl[$i + 2]->Des = 'em';    // Accusatif
                $this->Decl[$i + 3]->Des = 'is';    //Génitif
                $this->Decl[$i + 4]->Des = 'i';     // Datif
                $this->Decl[$i + 5]->Des = 'i';     // Ablatif
                }
            elseif ($d == 'Bns')
                {
                $this->Decl[$i + 0]->Rac = substr($Radic, 0, -1); // Nominatif
                $this->Decl[$i + 1]->Rac = substr($Radic, 0, -1); // Vocatif
                $this->Decl[$i + 2]->Rac = substr($Radic, 0, -1); // Accusatif
                
                $this->Decl[$i + 0]->Des = 's';     // Nominatif
                $this->Decl[$i + 1]->Des = 's';     // Vocatif
                $this->Decl[$i + 2]->Des = 's';     // Accusatif
                $this->Decl[$i + 3]->Des = 'is';    // Génitif
                $this->Decl[$i + 4]->Des = 'i';     // Datif
                $this->Decl[$i + 5]->Des = 'i';     // Ablatif
                }
            elseif ($d == 'Bex')
                {
                $this->Decl[$i + 0]->Rac = substr($Radic, 0, -2); // Nominatif
                $this->Decl[$i + 1]->Rac = substr($Radic, 0, -2); // Vocatif
                $this->Decl[$i + 2]->Rac = substr($Radic, 0, -2); // Accusatif
                
                $this->Decl[$i + 0]->Des = 'ex';    // Nominatif
                $this->Decl[$i + 1]->Des = 'ex';    // Vocatif
                $this->Decl[$i + 2]->Des = 'ex';    // Accusatif
                $this->Decl[$i + 3]->Des = 'is';    // Génitif
                $this->Decl[$i + 4]->Des = 'i';     // Datif
                $this->Decl[$i + 5]->Des = 'i';     // Ablatif
                }
            elseif ($d == 'Bx')
                {
                $this->Decl[$i + 0]->Rac = substr($Radic, 0, -1); // Nominatif
                $this->Decl[$i + 1]->Rac = substr($Radic, 0, -1); // Vocatif
                
                $this->Decl[$i + 0]->Des = 'x';     // Nominatif
                $this->Decl[$i + 1]->Des = 'x';     // Vocatif
                $this->Decl[$i + 2]->Des = 'e';     // Accusatif
                $this->Decl[$i + 3]->Des = 'is';    // Génitif
                $this->Decl[$i + 4]->Des = 'i';     // Datif
                $this->Decl[$i + 5]->Des = 'i';     // Ablatif
                }
            elseif ($d == 'Bor')
                {
                $this->Decl[$i + 0]->Des = '';      // Nominatif
                $this->Decl[$i + 1]->Des = '';      // Vocatif
                $this->Decl[$i + 2]->Des = '';      // Accusatif
                $this->Decl[$i + 3]->Des = 'is';    // Génitif
                $this->Decl[$i + 4]->Des = 'i';     // Datif
                $this->Decl[$i + 5]->Des = 'i';     // Ablatif
                }
            elseif ($d == 'Bus')
                {
                $this->Decl[$i + 0]->Rac = substr($Radic, 0, -2); // Nominatif
                $this->Decl[$i + 1]->Rac = substr($Radic, 0, -2); // Vocatif
                $this->Decl[$i + 2]->Rac = substr($Radic, 0, -2); // Accusatif
                
                $this->Decl[$i + 0]->Des = 'us';    // Nominatif
                $this->Decl[$i + 1]->Des = 'us';    // Vocatif
                $this->Decl[$i + 2]->Des = 'us';    // Accusatif
                $this->Decl[$i + 3]->Des = 'is';    // Génitif
                $this->Decl[$i + 4]->Des = 'i';     // Datif
                $this->Decl[$i + 5]->Des = 'e';     // Ablatif
                }
            elseif (in_array($d, Array('Ci', 'Co')))
                {
                $this->Decl[$i + 0]->Des = 'es';    // Nominatif
                $this->Decl[$i + 1]->Des = 'es';    // Vocatif
                $this->Decl[$i + 2]->Des = 'es';    // Accusatif
                $this->Decl[$i + 3]->Des = 'is';    // Génitif
                $this->Decl[$i + 4]->Des = 'i';     // Datif
                $this->Decl[$i + 5]->Des = 'e';     // Ablatif
                }
            elseif ($d == 'cmp')
                {
                $this->Decl[$i + 0]->Des = 'us';    // Nominatif
                $this->Decl[$i + 1]->Des = 'us';    // Vocatif
                $this->Decl[$i + 2]->Des = 'orem';  // Accusatif
                $this->Decl[$i + 3]->Des = 'oris';  // Génitif
                $this->Decl[$i + 4]->Des = 'ori';   // Datif
                $this->Decl[$i + 5]->Des = 'ore';   // Ablatif
                }
            elseif ($d == 'cmp-i') //(hors contexte, il ne semble y avoir que des pluriels : 'plures/a')
                {
                //pass;
                }
        
        
            }
        // ==========================================================
        if ($this->isPlural)    // === PLURIEL                          
        // ==========================================================
            {
            // -----------------------------------------------------
            // --- Masculin pluriel                                 
            // -----------------------------------------------------
            
            $i = 6; // index de départ dans Decl
            
            
            for ($n = 0; $n <= 5; $n++)        // (0 To 5) racine par défaut (= radical)
                {
                $this->Decl[$i + $n]->Rac = $Radic;
                }
            
            if (in_array($d, Array('A', 'Aa', 'Ab')))
                {
                $this->Decl[$i + 0]->Des = 'i';     // Nominatif
                $this->Decl[$i + 1]->Des = 'i';     // Vocatif
                $this->Decl[$i + 2]->Des = 'os';    // Accusatif
                $this->Decl[$i + 3]->Des = 'orum';  // Génitif
                $this->Decl[$i + 4]->Des = 'is';    // Datif
                $this->Decl[$i + 5]->Des = 'is';    // Ablatif
                }
            elseif (in_array($d, Array('Bp', 'Bns', 'Bex', 'Bx', 'Bor', 'Ber', 'cmp-i')))
                {
                $this->Decl[$i + 0]->Des = 'es';    // Nominatif
                $this->Decl[$i + 1]->Des = 'es';    // Vocatif
                $this->Decl[$i + 2]->Des = 'es';    // Accusatif
                $this->Decl[$i + 3]->Des = 'ium';   // Génitif
                $this->Decl[$i + 4]->Des = 'ibus';  // Datif
                $this->Decl[$i + 5]->Des = 'ibus';  // Ablatif
                }
            elseif ($d == 'Bus')
                {
                $this->Decl[$i + 0]->Des = 'es';    // Nominatif
                $this->Decl[$i + 1]->Des = 'es';    // Vocatif
                $this->Decl[$i + 2]->Des = 'es';    // Accusatif
                $this->Decl[$i + 3]->Des = 'um';    // Génitif
                $this->Decl[$i + 4]->Des = 'ibus';  // Datif
                $this->Decl[$i + 5]->Des = 'ibus';  // Ablatif
                }
            elseif (in_array($d, Array('Ci', 'Co')))
                {
                $this->Decl[$i + 0]->Des = 'es';    // Nominatif
                $this->Decl[$i + 1]->Des = 'es';    // Vocatif
                $this->Decl[$i + 2]->Des = 'es';    // Accusatif
                $this->Decl[$i + 3]->Des = 'um';    // Génitif
                $this->Decl[$i + 4]->Des = 'ibus';  // Datif
                $this->Decl[$i + 5]->Des = 'ibus';  // Ablatif
                }
            elseif ($d == 'cmp')
                {
                $this->Decl[$i + 0]->Des = 'ores';  // Nominatif
                $this->Decl[$i + 1]->Des = 'ores';  // Vocatif
                $this->Decl[$i + 2]->Des = 'ores';  // Accusatif
                $this->Decl[$i + 3]->Des = 'orum';  // Génitif
                $this->Decl[$i + 4]->Des = 'oribus';// Datif
                $this->Decl[$i + 5]->Des = 'oribus';// Ablatif
                }
            }
            
            // -----------------------------------------------------
            // --- Féminin pluriel                                  
            // -----------------------------------------------------
            
            $i = 18;    // index de départ dans Decl
            
            
            for ($n = 0; $n <= 5; $n++)        // (0 To 5) racine par défaut (= radical)
                {
                $this->Decl[$i + $n]->Rac = $Radic;
                }

            if (in_array($d, Array('A', 'Aa', 'Ab')))
                {
                $this->Decl[$i + 0]->Des = 'æ';    // Nominatif
                $this->Decl[$i + 1]->Des = 'æ';    // Vocatif
                $this->Decl[$i + 2]->Des = 'as';    // Accusatif
                $this->Decl[$i + 3]->Des = 'arum';  // Génitif
                $this->Decl[$i + 4]->Des = 'is';    // Datif
                $this->Decl[$i + 5]->Des = 'is';    // Ablatif
                }
            elseif (in_array($d, Array('Bp', 'Bns', 'Bex', 'Bx', 'Bor', 'Ber', 'cmp-i')))
                {
                $this->Decl[$i + 0]->Des = 'es';    // Nominatif
                $this->Decl[$i + 1]->Des = 'es';    // Vocatif
                $this->Decl[$i + 2]->Des = 'es';    // Accusatif
                $this->Decl[$i + 3]->Des = 'ium';   // Génitif
                $this->Decl[$i + 4]->Des = 'ibus';  // Datif
                $this->Decl[$i + 5]->Des = 'ibus';  // Ablatif
                }
            elseif ($d == 'Bus')
                {
                $this->Decl[$i + 0]->Des = 'es';    // Nominatif
                $this->Decl[$i + 1]->Des = 'es';    // Vocatif
                $this->Decl[$i + 2]->Des = 'es';    // Accusatif
                $this->Decl[$i + 3]->Des = 'um';    // Génitif
                $this->Decl[$i + 4]->Des = 'ibus';  // Datif
                $this->Decl[$i + 5]->Des = 'ibus';  // Ablatif
                }
            elseif (in_array($d, Array('Ci', 'Co')))
                {
                $this->Decl[$i + 0]->Des = 'es';    // Nominatif
                $this->Decl[$i + 1]->Des = 'es';    // Vocatif
                $this->Decl[$i + 2]->Des = 'es';    // Accusatif
                $this->Decl[$i + 3]->Des = 'um';    // Génitif
                $this->Decl[$i + 4]->Des = 'ibus';  // Datif
                $this->Decl[$i + 5]->Des = 'ibus';  // Ablatif
                }
            elseif ($d == 'cmp')
                {
                $this->Decl[$i + 0]->Des = 'ores';  // Nominatif
                $this->Decl[$i + 1]->Des = 'ores';  // Vocatif
                $this->Decl[$i + 2]->Des = 'ores';  // Accusatif
                $this->Decl[$i + 3]->Des = 'orum';  // Génitif
                $this->Decl[$i + 4]->Des = 'oribus';// Datif
                $this->Decl[$i + 5]->Des = 'oribus';// Ablatif
                }
            
            
            
            // -----------------------------------------------------
            // --- Neutre pluriel                                   
            // -----------------------------------------------------
            
            $i = 30;    // index de départ dans Decl
            
            
            for ($n = 0; $n <= 5; $n++)         // (0 To 5) racine par défaut (= radical)
                {
                $this->Decl[$i + $n]->Rac = $Radic;
                }
            
            if (in_array($d, Array('A', 'Aa', 'Ab')))
                {
                $this->Decl[$i + 0]->Des = 'a';     // Nominatif
                $this->Decl[$i + 1]->Des = 'a';     // Vocatif
                $this->Decl[$i + 2]->Des = 'a';     // Accusatif
                $this->Decl[$i + 3]->Des = 'orum';  // Génitif
                $this->Decl[$i + 4]->Des = 'is';    // Datif
                $this->Decl[$i + 5]->Des = 'is';    // Ablatif
                }
            elseif (in_array($d, Array('Bp', 'Bns', 'Bex', 'Bx', 'Bor', 'Ber')))
                {
                $this->Decl[$i + 0]->Des = 'ia';    // Nominatif
                $this->Decl[$i + 1]->Des = 'ia';    // Vocatif
                $this->Decl[$i + 2]->Des = 'ia';    // Accusatif
                $this->Decl[$i + 3]->Des = 'ium';   // Génitif
                $this->Decl[$i + 4]->Des = 'ibus';  // Datif
                $this->Decl[$i + 5]->Des = 'ibus';  // Ablatif
                }
            elseif ($d == 'Bus')
                {
                $this->Decl[$i + 0]->Des = 'a';     // Nominatif
                $this->Decl[$i + 1]->Des = 'a';     // Vocatif
                $this->Decl[$i + 2]->Des = 'a';     // Accusatif
                $this->Decl[$i + 3]->Des = 'um';    // Génitif
                $this->Decl[$i + 4]->Des = 'ibus';  // Datif
                $this->Decl[$i + 5]->Des = 'ibus';  // Ablatif
                }
            elseif (in_array($d, Array('Ci', 'Co')))
                {
                $this->Decl[$i + 0]->Des = 'a';     // Nominatif
                $this->Decl[$i + 1]->Des = 'a';     // Vocatif
                $this->Decl[$i + 2]->Des = 'a';     // Accusatif
                $this->Decl[$i + 3]->Des = 'um';    // Génitif
                $this->Decl[$i + 4]->Des = 'ibus';  // Datif
                $this->Decl[$i + 5]->Des = 'ibus';  // Ablatif
                }
            elseif ($d == 'cmp')
                {
                $this->Decl[$i + 0]->Des = 'ora';   // Nominatif
                $this->Decl[$i + 1]->Des = 'ora';   // Vocatif
                $this->Decl[$i + 2]->Des = 'ora';   // Accusatif
                $this->Decl[$i + 3]->Des = 'orum';  // Génitif
                $this->Decl[$i + 4]->Des = 'oribus';// Datif
                $this->Decl[$i + 5]->Des = 'oribus';// Ablatif
                }
            elseif ($d == 'cmp-i')
                {
                $this->Decl[$i + 0]->Des = 'a';     // Nominatif
                $this->Decl[$i + 1]->Des = 'a';     // Vocatif
                $this->Decl[$i + 2]->Des = 'a';     // Accusatif
                $this->Decl[$i + 3]->Des = 'ium';   // Génitif
                $this->Decl[$i + 4]->Des = 'ibus';  // Datif
                $this->Decl[$i + 5]->Des = 'ibus';  // Ablatif
                }
        
        
        
        }





/***************************************************************************//**
 */
    function DeclinerDemonstratif($mot)
        {
        
        $mot = $mot.lower();
        
        
        if ($mot == 'hic')
            {
            $this->isMasculine = True;
            $this->isFeminine = True;
            $this->isNeutral = True;
            
            $this->isDeclined = True;
            
            // ----------------------------------------------
            // --- Masculin singulier                        
            // ----------------------------------------------
            
            $i = 0;     // index de départ dans Decl
            
            for ($n = 0; $n <= 5; $n++)     // (0 To 5) racine par défaut (= radical)
                {
                $this->Decl[$i + $n]->Rac = 'h';
                }
                
            $this->Decl[$i + 1]->Rac = '';      // (pas de Vocatif)
            
            $this->Decl[$i + 0]->Des = 'ic';    // Nominatif
            $this->Decl[$i + 1]->Des = '';      // (pas de Vocatif)
            $this->Decl[$i + 2]->Des = 'unc';   // Accusatif
            $this->Decl[$i + 3]->Des = 'ujus';  // Génitif
            $this->Decl[$i + 4]->Des = 'uic';   // Datif
            $this->Decl[$i + 5]->Des = 'oc';    // Ablatif
            
            // --------------------------------------------
            // --- Féminin singulier                       
            // --------------------------------------------
            
            $i = 12;    // index de départ dans l'Array Decl
            
            for ($n = 0; $n <= 5; $n++)        // (0 To 5) racine par défaut (= radical)
                {
                $this->Decl[$i + $n]->Rac = 'h';
                }
            
            $this->Decl[$i + 1]->Rac = '';      // (pas de Vocatif)
            
            $this->Decl[$i + 0]->Des = 'aec';   // Nominatif
            $this->Decl[$i + 1]->Des = '';      // (pas de Vocatif)
            $this->Decl[$i + 2]->Des = 'anc';   // Accusatif
            $this->Decl[$i + 3]->Des = 'ujus';  // Génitif
            $this->Decl[$i + 4]->Des = 'uic';   // Datif
            $this->Decl[$i + 5]->Des = 'ac';    // Ablatif
            
            
            // --------------------------------------------
            // --- Neutre singulier                        
            // --------------------------------------------
            
            $i = 24;    // index de départ dans Decl
            
            for ($n = 0; $n <= 5; $n++)        // (0 To 5) racine par défaut (= radical)
                {
                $this->Decl[$i + $n]->Rac = 'h';
                }
            
            $this->Decl[$i + 1]->Rac = '';      // (pas de Vocatif)
            
            $this->Decl[$i + 0]->Des = 'oc';    // Nominatif
            $this->Decl[$i + 1]->Des = '';      // (pas de Vocatif)
            $this->Decl[$i + 2]->Des = 'oc';    // Accusatif
            $this->Decl[$i + 3]->Des = 'ujus';  // Génitif
            $this->Decl[$i + 4]->Des = 'uic';   // Datif
            $this->Decl[$i + 5]->Des = 'oc';    // Ablatif
            
            
            // -------------------------------------------
            // --- Masculin pluriel                       
            // -------------------------------------------
            
            $i = 6;     // index de départ dans Decl
            
            for ($n = 0; $n <= 5; $n++)        // (0 To 5) racine par défaut (= radical)
                {
                $this->Decl[$i + $n]->Rac = 'h';
                }
            
            $this->Decl[$i + 1]->Rac = '';        // (pas de Vocatif)
            
            $this->Decl[$i + 0]->Des = 'i';       // Nominatif
            $this->Decl[$i + 1]->Des = '';        // (pas de Vocatif) 
            $this->Decl[$i + 2]->Des = 'os';      // Accusatif
            $this->Decl[$i + 3]->Des = 'orum';    // Génitif
            $this->Decl[$i + 4]->Des = 'is';      // Datif
            $this->Decl[$i + 5]->Des = 'is';      // Ablatif
            
            // -----------------------------------------
            // --- Feminin pluriel                      
            // -----------------------------------------
            
            $i = 18;    // index de départ dans Decl
            
            for ($n = 0; $n <= 5; $n++)        // (0 To 5) racine par défaut (= radical)
                {
                $this->Decl[$i + $n]->Rac = 'h';
                }
            
            $this->Decl[$i + 1]->Rac = '';        // (pas de Vocatif)
            
            $this->Decl[$i + 0]->Des = 'æ';      // Nominatif
            $this->Decl[$i + 1]->Des = '';        // (pas de Vocatif)
            $this->Decl[$i + 2]->Des = 'as';      // Accusatif
            $this->Decl[$i + 3]->Des = 'arum';    // Génitif
            $this->Decl[$i + 4]->Des = 'is';      // Datif
            $this->Decl[$i + 5]->Des = 'is';      // Ablatif
            
            // -----------------------------------------
            // --- Neutre pluriel                       
            // -----------------------------------------
            
            $i = 30;    // index de départ dans Decl
            
            for ($n = 0; $n <= 5; $n++)        // (0 To 5) racine par défaut (= radical)
                {
                $this->Decl[$i + $n]->Rac = 'h';
                }
            
            $this->Decl[$i + 1]->Rac = '';        // (pas de Vocatif)
            
            $this->Decl[$i + 0]->Des = 'æc';     // Nominatif
            $this->Decl[$i + 1]->Des = '';        // (pas de Vocatif)
            $this->Decl[$i + 2]->Des = 'æc';     // Accusatif
            $this->Decl[$i + 3]->Des = 'orum';    // Génitif
            $this->Decl[$i + 4]->Des = 'is';      // Datif
            $this->Decl[$i + 5]->Des = 'is';      // Ablatif
            }
        
        return ;

        
        
        }




/***************************************************************************//**
 */
    function DeclinerNumeral($Radic, $d)
        {
        
        
        if ($d == '')
            {
            return ;
            }
            
        $i = 0;
        $n = 0;
        // ------------
        
        $this->isMasculine = True;
        $this->isFeminine = True;
        $this->isNeutral = True;
        
        $this->isDeclined = True;
        
        if ($d == 'num1') // =================================================
            {
            // ----------------------------------------------
            // --- Masculin singulier                        
            // ----------------------------------------------
            
            $i = 0;     // index de départ dans Decl
            
            for ($n = 0; $n <= 5; $n++)     // (0 To 5) racine par défaut (= radical)
                {
                $this->Decl[$i + $n]->Rac = $Radic;
                }
            
            $this->Decl[$i + 1]->Rac = '';      // (pas de Vocatif) 
            
            $this->Decl[$i + 0]->Des = 'us';    // Nominatif
            $this->Decl[$i + 1]->Des = '';      // (pas de Vocatif) 
            $this->Decl[$i + 2]->Des = 'um';    // Accusatif
            $this->Decl[$i + 3]->Des = 'ius';   // Génitif
            $this->Decl[$i + 4]->Des = 'i';     // Datif
            $this->Decl[$i + 5]->Des = 'o';     // Ablatif
            
            // --------------------------------------------
            // --- Féminin singulier                       
            // --------------------------------------------
            
            $i = 12;    // index de départ dans l'Array Decl
            
            for ($n = 0; $n <= 5; $n++)        // (0 To 5) racine par défaut (= radical)
                {
                $this->Decl[$i + $n]->Rac = $Radic;
                }
            
            
            $this->Decl[$i + 1]->Rac = '';      // (pas de Vocatif) 
            
            $this->Decl[$i + 0]->Des = 'a';     // Nominatif
            $this->Decl[$i + 1]->Des = '';      // (pas de Vocatif) 
            $this->Decl[$i + 2]->Des = 'am';    // Accusatif
            $this->Decl[$i + 3]->Des = 'ius';   // Génitif
            $this->Decl[$i + 4]->Des = 'i';     // Datif
            $this->Decl[$i + 5]->Des = 'a';     // Ablatif
            
            
            // --------------------------------------------
            // --- Neutre singulier                        
            // --------------------------------------------
            
            $i = 24;    // index de départ dans Decl
            
            for ($n = 0; $n <= 5; $n++)        // (0 To 5) racine par défaut (= radical)
                {
                $this->Decl[$i + $n]->Rac = $Radic;
                }
            
            $this->Decl[$i + 1]->Rac = '';      // (pas de Vocatif) 
            
            $this->Decl[$i + 0]->Des = 'um';    // Nominatif
            $this->Decl[$i + 1]->Des = '';      // (pas de Vocatif) 
            $this->Decl[$i + 2]->Des = 'um';    // Accusatif
            $this->Decl[$i + 3]->Des = 'ius';   // Génitif
            $this->Decl[$i + 4]->Des = 'i';     // Datif
            $this->Decl[$i + 5]->Des = 'o';     // Ablatif
            
            
            // -------------------------------------------
            // --- Masculin pluriel                       
            // -------------------------------------------
            
            $i = 6;     // index de départ dans Decl
            
            for ($n = 0; $n <= 5; $n++)        // (0 To 5) racine par défaut (= radical)
                {
                $this->Decl[$i + $n]->Rac = $Radic;
                }
            
            $this->Decl[$i + 1]->Rac = '';        // (pas de Vocatif) 
            
            $this->Decl[$i + 0]->Des = 'i';       // Nominatif
            $this->Decl[$i + 1]->Des = '';        // (pas de Vocatif) 
            $this->Decl[$i + 2]->Des = 'os';      // Accusatif
            $this->Decl[$i + 3]->Des = 'orum';    // Génitif
            $this->Decl[$i + 4]->Des = 'is';      // Datif
            $this->Decl[$i + 5]->Des = 'is';      // Ablatif
            
            // -----------------------------------------
            // --- Feminin pluriel                      
            // -----------------------------------------
            
            $i = 18;    // index de départ dans Decl
            
            for ($n = 0; $n <= 5; $n++)        // (0 To 5) racine par défaut (= radical)
                {
                $this->Decl[$i + $n]->Rac = $Radic;
                }
            
            $this->Decl[$i + 1]->Rac = '';        // (pas de Vocatif) 
            
            $this->Decl[$i + 0]->Des = 'æ';      // Nominatif
            $this->Decl[$i + 1]->Des = '';        // (pas de Vocatif) 
            $this->Decl[$i + 2]->Des = 'as';      // Accusatif
            $this->Decl[$i + 3]->Des = 'arum';    // Génitif
            $this->Decl[$i + 4]->Des = 'is';      // Datif
            $this->Decl[$i + 5]->Des = 'is';      // Ablatif
            
            // -----------------------------------------
            // --- Neutre pluriel                       
            // -----------------------------------------
            
            $i = 30;    // index de départ dans Decl
            
            for ($n = 0; $n <= 5; $n++)        // (0 To 5) racine par défaut (= radical)
                {
                $this->Decl[$i + $n]->Rac = $Radic;
                }
            
            $this->Decl[$i + 1]->Rac = '';        // (pas de Vocatif) 
            
            $this->Decl[$i + 0]->Des = 'a';       // Nominatif
            $this->Decl[$i + 1]->Des = '';        // (pas de Vocatif) 
            $this->Decl[$i + 2]->Des = 'a';       // Accusatif
            $this->Decl[$i + 3]->Des = 'orum';    // Génitif
            $this->Decl[$i + 4]->Des = 'is';      // Datif
            $this->Decl[$i + 5]->Des = 'is';      // Ablatif
        
        
            }
        elseif ($d == 'num2') // ====================================================
            {
            // -------------------------------------------
            // --- Masculin pluriel                       
            // -------------------------------------------
            
            $i = 6;     // index de départ dans Decl
            
            for ($n = 0; $n <= 5; $n++)        // (0 To 5) racine par défaut (= radical)
                {
                $this->Decl[$i + $n]->Rac = $Radic;
                }
            
            $this->Decl[$i + 1]->Rac = '';        // (pas de Vocatif) 
            
            $this->Decl[$i + 0]->Des = 'o';       // Nominatif
            $this->Decl[$i + 1]->Des = '';        // (pas de Vocatif) 
            $this->Decl[$i + 2]->Des = 'os/o';  // Accusatif
            $this->Decl[$i + 3]->Des = 'orum';    // Génitif
            $this->Decl[$i + 4]->Des = 'obus';    // Datif
            $this->Decl[$i + 5]->Des = 'obus';    // Ablatif
            
            // ------------------------------------------
            // --- Féminin pluriel                       
            // ------------------------------------------
            
            $i = 18;    // index de départ dans Decl
            
            for ($n = 0; $n <= 5; $n++)        // (0 To 5) racine par défaut (= radical)
                {
                $this->Decl[$i + $n]->Rac = $Radic;
                }
            
            $this->Decl[$i + 1]->Rac = '';        // (pas de Vocatif) 
            
            $this->Decl[$i + 0]->Des = 'æ';      // Nominatif
            $this->Decl[$i + 1]->Des = '';        // (pas de Vocatif) 
            $this->Decl[$i + 2]->Des = 'as';      // Accusatif
            $this->Decl[$i + 3]->Des = 'arum';    // Génitif
            $this->Decl[$i + 4]->Des = 'abus';    // Datif
            $this->Decl[$i + 5]->Des = 'abus';    // Ablatif
            
            // -----------------------------------------
            // --- Neutre pluriel                       
            // -----------------------------------------
            
            $i = 30;    // index de départ dans Decl
            
            for ($n = 0; $n <= 5; $n++)        // (0 To 5) racine par défaut (= radical)
                {
                $this->Decl[$i + $n]->Rac = $Radic;
                }
            
            $this->Decl[$i + 1]->Rac = '';        // (pas de Vocatif) 
            
            $this->Decl[$i + 0]->Des = 'o';       // Nominatif
            $this->Decl[$i + 1]->Des = '';        // (pas de Vocatif) 
            $this->Decl[$i + 2]->Des = 'o';       // Accusatif
            $this->Decl[$i + 3]->Des = 'orum';    // Génitif
            $this->Decl[$i + 4]->Des = 'obus';    // Datif
            $this->Decl[$i + 5]->Des = 'obus';    // Ablatif
        
        
        
            }
        elseif ($d == 'num3') // ============================================
            {
            // ----------------------------------------
            // --- Masculin pluriel                    
            // ----------------------------------------
            
            $i = 6;     // index de départ dans Decl
            
            for ($n = 0; $n <= 5; $n++)        // (0 To 5) racine par défaut (= radical)
                {
                $this->Decl[$i + $n]->Rac = $Radic;
                }
            
            $this->Decl[$i + 1]->Rac = '';        // (pas de Vocatif) 
            
            $this->Decl[$i + 0]->Des = 'es';      // Nominatif
            $this->Decl[$i + 1]->Des = '';        // (pas de Vocatif) 
            $this->Decl[$i + 2]->Des = 'es';      // Accusatif
            $this->Decl[$i + 3]->Des = 'ium';     // Génitif
            $this->Decl[$i + 4]->Des = 'ibus';    // Datif
            $this->Decl[$i + 5]->Des = 'ibus';    // Ablatif
            
            // -----------------------------------------
            // --- Féminin pluriel                      
            // -----------------------------------------
            
            $i = 18;    // index de départ dans l'Array Decl
            
            for ($n = 0; $n <= 5; $n++)        // (0 To 5) racine par défaut (= radical)
                {
                $this->Decl[$i + $n]->Rac = $Radic;
                }
            
            $this->Decl[$i + 1]->Rac = '';        // (pas de Vocatif) 
            
            $this->Decl[$i + 0]->Des = 'es';      // Nominatif
            $this->Decl[$i + 1]->Des = '';        // (pas de Vocatif) 
            $this->Decl[$i + 2]->Des = 'es';      // Accusatif
            $this->Decl[$i + 3]->Des = 'ium';     // Génitif
            $this->Decl[$i + 4]->Des = 'ibus';    // Datif
            $this->Decl[$i + 5]->Des = 'ibus';    // Ablatif
            
            // ----------------------------------------
            // --- Neutre pluriel                      
            // ----------------------------------------
            
            $i = 30;    // index de départ dans Decl
            
            for ($n = 0; $n <= 5; $n++)        // (0 To 5) racine par défaut (= radical)
                {
                $this->Decl[$i + $n]->Rac = $Radic;
                }
            
            $this->Decl[$i + 1]->Rac = '';        // (pas de Vocatif) 
            
            $this->Decl[$i + 0]->Des = 'ia';      // Nominatif
            $this->Decl[$i + 1]->Des = '';        // (pas de Vocatif) 
            $this->Decl[$i + 2]->Des = 'ia';      // Accusatif
            $this->Decl[$i + 3]->Des = 'ium';     // Génitif
            $this->Decl[$i + 4]->Des = 'ibus';    // Datif
            $this->Decl[$i + 5]->Des = 'ibus';    // Ablatif
        
            }
        else
            {
            $this->isDeclined = False;
            }
        
        return ;
        
        
        }




/***************************************************************************//**
 */
    function declinerRelatif()
        {
        
        $this->isMasculine = True;
        $this->isFeminine = True;
        $this->isNeutral = True;
        
        $this->isDeclined = True;
        
        // ----------------------------------------------
        // --- Masculin singulier                        
        // ----------------------------------------------
        
        $i = 0;     // index de départ dans Decl
        
        for ($n = 0; $n <= 5; $n++)     // (0 To 5) racine par défaut (= radical)
            {
            $this->Decl[$i + $n]->Rac = 'qu';
            }
            
        $this->Decl[$i + 1]->Rac = '';      // (pas de Vocatif)
        $this->Decl[$i + 3]->Rac = 'cu';    // Génitif
        $this->Decl[$i + 4]->Rac = 'cu';    // Datif
        
        $this->Decl[$i + 0]->Des = 'i';     // Nominatif
        $this->Decl[$i + 1]->Des = '';      // (pas de Vocatif)
        $this->Decl[$i + 2]->Des = 'em';    // Accusatif
        $this->Decl[$i + 3]->Des = 'jus';   // Génitif
        $this->Decl[$i + 4]->Des = 'i';     // Datif
        $this->Decl[$i + 5]->Des = 'o';     // Ablatif
        
        // --------------------------------------------
        // --- Féminin singulier                       
        // --------------------------------------------
        
        $i = 12;    // index de départ dans Decl
        
        for ($n = 0; $n <= 5; $n++)        // (0 To 5) racine par défaut (= radical)
            {
            $this->Decl[$i + $n]->Rac = 'qu';
            }
        
        $this->Decl[$i + 1]->Rac = '';      // (pas de Vocatif)
        $this->Decl[$i + 3]->Rac = 'cu';    // Génitif
        $this->Decl[$i + 4]->Rac = 'cu';    // Datif
        
        $this->Decl[$i + 0]->Des = 'ae';    // Nominatif
        $this->Decl[$i + 1]->Des = '';      // (pas de Vocatif)
        $this->Decl[$i + 2]->Des = 'am';    // Accusatif
        $this->Decl[$i + 3]->Des = 'jus';   // Génitif
        $this->Decl[$i + 4]->Des = 'i';     // Datif
        $this->Decl[$i + 5]->Des = 'a';     // Ablatif
        
        
        // --------------------------------------------
        // --- Neutre singulier                        
        // --------------------------------------------
        
        $i = 24;    // index de départ dans Decl
        
        for ($n = 0; $n <= 5; $n++)        // (0 To 5) racine par défaut (= radical)
            {
            $this->Decl[$i + $n]->Rac = 'qu';
            }
            
        $this->Decl[$i + 1]->Rac = '';      // (pas de Vocatif)
        $this->Decl[$i + 3]->Rac = 'cu';    // Génitif
        $this->Decl[$i + 4]->Rac = 'cu';    // Datif
        
        $this->Decl[$i + 0]->Des = 'od';    // Nominatif
        $this->Decl[$i + 1]->Des = '';      // (pas de Vocatif)
        $this->Decl[$i + 2]->Des = 'od';    // Accusatif
        $this->Decl[$i + 3]->Des = 'jus';  // Génitif
        $this->Decl[$i + 4]->Des = 'i';   // Datif
        $this->Decl[$i + 5]->Des = 'o';    // Ablatif
        
        
        // -------------------------------------------
        // --- Masculin pluriel                       
        // -------------------------------------------
        
        $i = 6;     // index de départ dans Decl
        
        for ($n = 0; $n <= 5; $n++)        // (0 To 5) racine par défaut (= radical)
            {
            $this->Decl[$i + $n]->Rac = 'qu';
            }
            
        $this->Decl[$i + 1]->Rac = '';        // (pas de Vocatif)
        
        $this->Decl[$i + 0]->Des = 'i';       // Nominatif
        $this->Decl[$i + 1]->Des = '';        // (pas de Vocatif) 
        $this->Decl[$i + 2]->Des = 'os';      // Accusatif
        $this->Decl[$i + 3]->Des = 'orum';    // Génitif
        $this->Decl[$i + 4]->Des = 'ibus';    // Datif
        $this->Decl[$i + 5]->Des = 'ibus';    // Ablatif
        
        // -----------------------------------------
        // --- Feminin pluriel                      
        // -----------------------------------------
        
        $i = 18;    // index de départ dans Decl
        
        for ($n = 0; $n <= 5; $n++)        // (0 To 5) racine par défaut (= radical)
            {
            $this->Decl[$i + $n]->Rac = 'qu';
            }
        
        $this->Decl[$i + 1]->Rac = '';        // (pas de Vocatif)
        
        $this->Decl[$i + 0]->Des = 'æ';      // Nominatif
        $this->Decl[$i + 1]->Des = '';        // (pas de Vocatif)
        $this->Decl[$i + 2]->Des = 'as';      // Accusatif
        $this->Decl[$i + 3]->Des = 'arum';    // Génitif
        $this->Decl[$i + 4]->Des = 'ibus';    // Datif
        $this->Decl[$i + 5]->Des = 'ibus';    // Ablatif
        
        // -----------------------------------------
        // --- Neutre pluriel                       
        // -----------------------------------------
        
        $i = 30;    // index de départ dans Decl
        
        for ($n = 0; $n <= 5; $n++)        // (0 To 5) racine par défaut (= radical)
            {
            $this->Decl[$i + $n]->Rac = 'qu';
            }
        
        $this->Decl[$i + 1]->Rac = '';      // (pas de Vocatif)
        
        $this->Decl[$i + 0]->Des = 'æ';    // Nominatif
        $this->Decl[$i + 1]->Des = '';      // (pas de Vocatif)
        $this->Decl[$i + 2]->Des = 'æ';    // Accusatif
        $this->Decl[$i + 3]->Des = 'orum';  // Génitif
        $this->Decl[$i + 4]->Des = 'ibus';  // Datif
        $this->Decl[$i + 5]->Des = 'ibus';  // Ablatif
        
        
        return ;
        
        
        }




/***************************************************************************//**
 * Décline le substantif de radical 'Radic' suivant la décinaison 'd'
 *  
 */
    function DeclinerSubstantif($Radic, $d)
        {
        
        
        
        if ($d == '')
            {
            return;
            }
        
        $i = 0;
        $flagGenreConnu = False;
        
        // -------------
        
        // =======================================================================
        if ($this->isMasculine)        // === Masculin                               
        // =======================================================================
            {
            $flagGenreConnu = True;
            
            if ($this->isSingular)  // --- singulier
                {
                $i = 0;
                $this->DeclinerSubstantif_Singulier($Radic, $d, $i);
                }
            
            if ($this->isPlural)    // --- pluriel
                {
                $i = 6;
                $this->Declinersubstantif_Pluriel($Radic, $d, $i);
                }
            }
        
        
        // =======================================================================
        if ($this->isFeminine)         // === Féminin                                
        // =======================================================================
            {
            $flagGenreConnu = True;
            
            if ($this->isSingular)  // --- singulier
                {
                $i = 12;
                $this->DeclinerSubstantif_Singulier($Radic, $d, $i);
                }
            
            if ($this->isPlural)    // --- pluriel
                {
                $i = 18;
                $this->Declinersubstantif_Pluriel($Radic, $d, $i);
                }
            }
        
        
        // ========================================================================
        if ($this->isNeutral)          // === Neutre                                  
        // ========================================================================
            {
            $flagGenreConnu = True;
            
            if ($this->isSingular)  // --- singulier
                {
                $i = 24;
                $this->DeclinerSubstantif_Singulier($Radic, $d, $i);
                }
            
            if ($this->isPlural)    // --- pluriel
                {
                $i = 30;
                $this->Declinersubstantif_Pluriel($Radic, $d, $i);
                }
            }
            
        
        // ===========================================================================
        if ($flagGenreConnu == False)  // genre non précisé > remplir comme masculin    
        // ===========================================================================
            {
            if ($this->isSingular)  // --- singulier
                {
                $i = 0;
                $this->DeclinerSubstantif_Singulier($Radic, $d, $i);
                }
            
            if ($this->isPlural)    // --- pluriel
                {
                $i = 6;
                $this->Declinersubstantif_Pluriel($Radic, $d, $i);
                }
            }
            
        return;

        
        
        
        }



/***************************************************************************//**
 * (sous-routine de DeclinerSubstantif)
 *       le paramètre i est l'index de départ dans Decl
 *
 *  
 */
    function Declinersubstantif_Pluriel($Radic, $d, $i)
        {
    
        
        $this->isDeclined = True;
        
        for ($n = 0; $n <= 5; $n++)     // (0 To 5) racine par défaut (= radical)
            {
            $this->Decl[$i + $n]->Rac = $Radic;
            }
            
        // --- 1ère déclinaison
        
        if (in_array($d, Array('1', '1e', '1ae')))
            {
            $this->Decl[$i + 0]->Des = 'æ';     // Nominatif
            $this->Decl[$i + 1]->Des = 'æ';     // Vocatif
            $this->Decl[$i + 2]->Des = 'as';     // Accusatif
            $this->Decl[$i + 3]->Des = 'arum';   // Génitif
            $this->Decl[$i + 4]->Des = 'is';     // Datif
            $this->Decl[$i + 5]->Des = 'is';     // Ablatif
            }
            
        // --- 2ème déclinaison
        
        elseif ($d == '2_mf')
            {
            $this->Decl[$i + 0]->Des = 'i';      // Nominatif
            $this->Decl[$i + 1]->Des = 'i';      // Vocatif
            $this->Decl[$i + 2]->Des = 'os';     // Accusatif
            $this->Decl[$i + 3]->Des = 'orum';   // Génitif
            $this->Decl[$i + 4]->Des = 'is';     // Datif
            $this->Decl[$i + 5]->Des = 'is';    // Ablatif
            }
        elseif ($d == '2_n')
            {
            $this->Decl[$i + 0]->Des = 'a';     // Nominatif
            $this->Decl[$i + 1]->Des = 'a';     // Vocatif
            $this->Decl[$i + 2]->Des = 'a';     // Accusatif
            $this->Decl[$i + 3]->Des = 'orum';  // Génitif
            $this->Decl[$i + 4]->Des = 'is';    // Datif
            $this->Decl[$i + 5]->Des = 'is';    // Ablatif
            }
        elseif (in_array($d, Array('2r1', '2r2')))
            {
            $this->Decl[$i + 0]->Des = 'i';     // Nominatif
            $this->Decl[$i + 1]->Des = 'i';     // Vocatif
            $this->Decl[$i + 2]->Des = 'os';    // Accusatif
            $this->Decl[$i + 3]->Des = 'orum';  // Génitif
            $this->Decl[$i + 4]->Des = 'is';    // Datif
            $this->Decl[$i + 5]->Des = 'is';    // Ablatif
            }
        elseif ($d == '2on')
            {
            $this->Decl[$i + 0]->Des = 'a';     // Nominatif
            $this->Decl[$i + 1]->Des = 'a';     // Vocatif
            $this->Decl[$i + 2]->Des = 'a';     // Accusatif
            $this->Decl[$i + 3]->Des = 'orum';  // Génitif
            $this->Decl[$i + 4]->Des = 'is';    // Datif
            $this->Decl[$i + 5]->Des = 'is';    // Ablatif
            }
        
        // --- 3ème déclinaison
        
        elseif (in_array($d, Array('3p_mf', '3pg', '3ns1', '3ns2')))
            {
            $this->Decl[$i + 0]->Des = 'es';    // Nominatif
            $this->Decl[$i + 1]->Des = 'es';    // Vocatif
            $this->Decl[$i + 2]->Des = 'es';    // Accusatif
            $this->Decl[$i + 3]->Des = 'ium';   // Génitif
            $this->Decl[$i + 4]->Des = 'ibus';  // Datif
            $this->Decl[$i + 5]->Des = 'ibus';  // Ablatif
            }
        elseif ($d == '3p_n')
            {
            $this->Decl[$i + 0]->Des = 'ia';    // Nominatif
            $this->Decl[$i + 1]->Des = 'ia';    // Vocatif
            $this->Decl[$i + 2]->Des = 'ia';    // Accusatif
            $this->Decl[$i + 3]->Des = 'ium';   // Génitif
            $this->Decl[$i + 4]->Des = 'ibus';  // Datif
            $this->Decl[$i + 5]->Des = 'ibus';  // Ablatif
            }
        elseif (in_array($d, Array('3a', '3en', '3er1', '3us1', '3us2')))
            {
            $this->Decl[$i + 0]->Des = 'a';     // Nominatif
            $this->Decl[$i + 1]->Des = 'a';     // Vocatif
            $this->Decl[$i + 2]->Des = 'a';     // Accusatif
            $this->Decl[$i + 3]->Des = 'um';    // Génitif
            $this->Decl[$i + 4]->Des = 'ibus';  // Datif
            $this->Decl[$i + 5]->Des = 'ibus';  // Ablatif
            }
        elseif (in_array($d, Array('3on', '3o1', '3o2', '3er2', '3or', '3as', '3es1', '3es2', '3is', '3isg', '3os', '3ax', '3ex', '3ix', '3s')))
            {
            $this->Decl[$i + 0]->Des = 'es';    // Nominatif
            $this->Decl[$i + 1]->Des = 'es';    // Vocatif
            $this->Decl[$i + 2]->Des = 'es';    // Accusatif
            $this->Decl[$i + 3]->Des = 'um';    // Génitif
            $this->Decl[$i + 4]->Des = 'ibus';  // Datif
            $this->Decl[$i + 5]->Des = 'ibus';  // Ablatif
            }
        elseif ($d == '3ar')
            {
            $this->Decl[$i + 0]->Des = 'ia';    // Nominatif
            $this->Decl[$i + 1]->Des = 'ia';    // Vocatif
            $this->Decl[$i + 2]->Des = 'ia';    // Accusatif
            $this->Decl[$i + 3]->Des = 'ium';   // Génitif
            $this->Decl[$i + 4]->Des = 'ibus';  // Datif
            $this->Decl[$i + 5]->Des = 'ibus';  // Ablatif
            }
            
        // --- 4ème déclinaison
        
        elseif ($d == '4_mf')
            {
            $this->Decl[$i + 0]->Des = 'us';    // Nominatif
            $this->Decl[$i + 1]->Des = 'us';    // Vocatif
            $this->Decl[$i + 2]->Des = 'us';    // Accusatif
            $this->Decl[$i + 3]->Des = 'uum';   // Génitif
            $this->Decl[$i + 4]->Des = 'ibus';  // Datif
            $this->Decl[$i + 5]->Des = 'ibus';  // Ablatif
            }
        elseif ($d == '4_n')
            {
            $this->Decl[$i + 0]->Des = 'ua';    // Nominatif
            $this->Decl[$i + 1]->Des = 'ua';    // Vocatif
            $this->Decl[$i + 2]->Des = 'ua';    // Accusatif
            $this->Decl[$i + 3]->Des = 'uum';   // Génitif
            $this->Decl[$i + 4]->Des = 'ibus';  // Datif
            $this->Decl[$i + 5]->Des = 'ibus';  // Ablatif
            }
        elseif ($d == '4a')
            {
            $this->Decl[$i + 0]->Des = 'us';    // Nominatif
            $this->Decl[$i + 1]->Des = 'us';    // Vocatif
            $this->Decl[$i + 2]->Des = 'us';    // Accusatif
            $this->Decl[$i + 3]->Des = 'uum';   // Génitif
            $this->Decl[$i + 4]->Des = 'ubus';  // Datif
            $this->Decl[$i + 5]->Des = 'ubus';  // Ablatif
            }
            
        // --- 5ème déclinaison
        
        elseif ($d == '5')
            {
            $this->Decl[$i + 0]->Des = 'es';    // Nominatif
            $this->Decl[$i + 1]->Des = 'es';    // Vocatif
            $this->Decl[$i + 2]->Des = 'es';    // Accusatif
            $this->Decl[$i + 3]->Des = 'erum';  // Génitif
            $this->Decl[$i + 4]->Des = 'ebus';  // Datif
            $this->Decl[$i + 5]->Des = 'ebus';  // Ablatif
            }
            
        // ---
        
        else
            {
            $this->isDeclined = False;
            }
        
        return;

    
    
        }



/***************************************************************************//**
 * (sous-routine de DeclinerSubstantif)
 *       le paramètre i est l'index de départ dans Decl
 * 
 */
    function DeclinerSubstantif_Singulier($Radic, $d, $i)
        {
    
        
        $this->isDeclined = True;
        
        for ($n = 0; $n <= 5; $n++)     // (0 To 5) racine par défaut (= radical)
            {
            $this->Decl[$i + $n]->Rac = $Radic;
            }
            
        if ($d == '1')
            {
            $this->Decl[$i + 0]->Des = 'a';       // Nominatif
            $this->Decl[$i + 1]->Des = 'a';       // Vocatif
            $this->Decl[$i + 2]->Des = 'am';      // Accusatif
            $this->Decl[$i + 3]->Des = 'æ';      // Génitif
            $this->Decl[$i + 4]->Des = 'æ';      // Datif
            $this->Decl[$i + 5]->Des = 'a';       // Ablatif
            }
        elseif ($d == '1e')
            {
            $this->Decl[$i + 0]->Des = 'a/e';     // Nominatif
            $this->Decl[$i + 1]->Des = '...';     // Vocatif
            $this->Decl[$i + 2]->Des = 'en';      // Accusatif
            $this->Decl[$i + 3]->Des = 'es';      // Génitif
            $this->Decl[$i + 4]->Des = 'æ';      // Datif
            $this->Decl[$i + 5]->Des = 'e';       // Ablatif
            }
        elseif ($d == '1ae')
            {
            $this->Decl[$i + 0]->Des = 'a';       // Nominatif
            $this->Decl[$i + 1]->Des = 'a';       // Vocatif
            $this->Decl[$i + 2]->Des = 'am';      // Accusatif
            $this->Decl[$i + 3]->Des = 'æ';      // Génitif
            $this->Decl[$i + 4]->Des = 'æ';      // Datif
            $this->Decl[$i + 5]->Des = 'e/a';     // Ablatif
            }
            
        // --- 2ème déclinaison
        
        elseif ($d == '2_mf')
            {
            $this->Decl[$i + 0]->Des = 'us';      // Nominatif
            $this->Decl[$i + 1]->Des = 'e';       // Vocatif
            $this->Decl[$i + 2]->Des = 'um';      // Accusatif
            $this->Decl[$i + 3]->Des = 'i';       // Génitif
            $this->Decl[$i + 4]->Des = 'o';       // Datif
            $this->Decl[$i + 5]->Des = 'o';       // Ablatif
            }
        elseif ($d == '2_n')
            {
            $this->Decl[$i + 0]->Des = 'um';      // Nominatif
            $this->Decl[$i + 1]->Des = 'um';      // Vocatif
            $this->Decl[$i + 2]->Des = 'um';      // Accusatif
            $this->Decl[$i + 3]->Des = 'i';       // Génitif
            $this->Decl[$i + 4]->Des = 'o';       // Datif
            $this->Decl[$i + 5]->Des = 'o';       // Ablatif
            }
        elseif ($d == '2r1')
            {
            $this->Decl[$i + 0]->Rac = substr($Radic, 0, -1) . 'er';// Nominatif
            $this->Decl[$i + 1]->Rac = substr($Radic, 0, -1) . 'er';// Vocatif
            
            $this->Decl[$i + 0]->Des = '';        // Nominatif
            $this->Decl[$i + 1]->Des = '';        // Vocatif
            $this->Decl[$i + 2]->Des = 'um';      // Accusatif
            $this->Decl[$i + 3]->Des = 'i';       // Génitif
            $this->Decl[$i + 4]->Des = 'o';       // Datif
            $this->Decl[$i + 5]->Des = 'o';       // Ablatif
            }
        elseif ($d == '2r2')
            {
            $this->Decl[$i + 0]->Des = '';        // Nominatif
            $this->Decl[$i + 1]->Des = '';        // Vocatif
            $this->Decl[$i + 2]->Des = 'um';      // Accusatif
            $this->Decl[$i + 3]->Des = 'i';       // Génitif
            $this->Decl[$i + 4]->Des = 'o';       // Datif
            $this->Decl[$i + 5]->Des = 'o';       // Ablatif
            }
        elseif ($d == '2on')
            {
            $this->Decl[$i + 0]->Des = 'on';      // Nominatif
            $this->Decl[$i + 1]->Des = 'on';      // Vocatif
            $this->Decl[$i + 2]->Des = 'on';      // Accusatif
            $this->Decl[$i + 3]->Des = 'i';       // Génitif
            $this->Decl[$i + 4]->Des = 'o';       // Datif
            $this->Decl[$i + 5]->Des = 'o';       // Ablatif
            }
            
        // --- 3ème déclinaison
        
        elseif ($d == '3p_mf')
            {
            $this->Decl[$i + 0]->Des = 'is';      // Nominatif
            $this->Decl[$i + 1]->Des = 'is';      // Vocatif
            $this->Decl[$i + 2]->Des = 'em';      // Accusatif
            $this->Decl[$i + 3]->Des = 'is';      // Génitif
            $this->Decl[$i + 4]->Des = 'i';       // Datif
            $this->Decl[$i + 5]->Des = 'e';       // Ablatif
            }
        elseif ($d == '3p_n')
            {
            $this->Decl[$i + 0]->Des = 'e';       // Nominatif
            $this->Decl[$i + 1]->Des = 'e';       // Vocatif
            $this->Decl[$i + 2]->Des = 'e';       // Accusatif
            $this->Decl[$i + 3]->Des = 'is';      // Génitif
            $this->Decl[$i + 4]->Des = 'i';       // Datif
            $this->Decl[$i + 5]->Des = 'i';       // Ablatif
            }
        elseif ($d == '3pg')
            {
            $this->Decl[$i + 0]->Des = 'is';      // Nominatif
            $this->Decl[$i + 1]->Des = 'is';      // Vocatif
            $this->Decl[$i + 2]->Des = 'in/im/em';// Accusatif
            $this->Decl[$i + 3]->Des = 'is/eos';  // Génitif
            $this->Decl[$i + 4]->Des = 'i';       // Datif
            $this->Decl[$i + 5]->Des = 'i/e';     // Ablatif
            }
        elseif ($d == '3a')
            {
            $this->Decl[$i + 0]->Rac = substr($Radic, 0, -1); // Nominatif
            $this->Decl[$i + 1]->Rac = substr($Radic, 0, -1); // Vocatif
            $this->Decl[$i + 2]->Rac = substr($Radic, 0, -1); // Accusatif
            
            $this->Decl[$i + 0]->Des = '';        // Nominatif
            $this->Decl[$i + 1]->Des = '';        // Vocatif
            $this->Decl[$i + 2]->Des = '';        // Accusatif
            $this->Decl[$i + 3]->Des = 'is';      // Génitif
            $this->Decl[$i + 4]->Des = 'i';       // Datif
            $this->Decl[$i + 5]->Des = 'e';       // Ablatif
            }
        elseif ($d == '3en')
            {
            $this->Decl[$i + 0]->Rac = substr($Radic, 0, -2); // Nominatif
            $this->Decl[$i + 1]->Rac = substr($Radic, 0, -2); // Vocatif
            $this->Decl[$i + 2]->Rac = substr($Radic, 0, -2); // Accusatif
            
            $this->Decl[$i + 0]->Des = 'en';      // Nominatif
            $this->Decl[$i + 1]->Des = 'en';      // Vocatif
            $this->Decl[$i + 2]->Des = 'en';      // Accusatif
            $this->Decl[$i + 3]->Des = 'is';      // Génitif
            $this->Decl[$i + 4]->Des = 'i';       // Datif
            $this->Decl[$i + 5]->Des = 'e';       // Ablatif
            }
        elseif (in_array($d, Array('3on', '3or', '3er2')))
            {
            $this->Decl[$i + 0]->Des = '';        // Nominatif
            $this->Decl[$i + 1]->Des = '';        // Vocatif
            $this->Decl[$i + 2]->Des = 'em';      // Accusatif
            $this->Decl[$i + 3]->Des = 'is';      // Génitif
            $this->Decl[$i + 4]->Des = 'i';       // Datif
            $this->Decl[$i + 5]->Des = 'e';       // Ablatif
            }
        elseif ($d == '3o1')
            {
            $this->Decl[$i + 0]->Rac = substr($Radic, 0, -1); // Nominatif
            $this->Decl[$i + 1]->Rac = substr($Radic, 0, -1); // Vocatif
            
            $this->Decl[$i + 0]->Des = '';        // Nominatif
            $this->Decl[$i + 1]->Des = '';        // Vocatif
            $this->Decl[$i + 2]->Des = 'em';      // Accusatif
            $this->Decl[$i + 3]->Des = 'is';      // Génitif
            $this->Decl[$i + 4]->Des = 'i';       // Datif
            $this->Decl[$i + 5]->Des = 'e';       // Ablatif
            }
        elseif ($d == '3o2')
            {
            $this->Decl[$i + 0]->Rac = substr($Radic, 0, -2); // Nominatif
            $this->Decl[$i + 1]->Rac = substr($Radic, 0, -2); // Vocatif
            
            $this->Decl[$i + 0]->Des = 'o';       // Nominatif
            $this->Decl[$i + 1]->Des = 'o';       // Vocatif
            $this->Decl[$i + 2]->Des = 'em';      // Accusatif
            $this->Decl[$i + 3]->Des = 'is';      // Génitif
            $this->Decl[$i + 4]->Des = 'i';       // Datif
            $this->Decl[$i + 5]->Des = 'e';       // Ablatif
            }
        elseif ($d == '3ar')
            {
            $this->Decl[$i + 0]->Des = '';        // Nominatif
            $this->Decl[$i + 1]->Des = '';        // Vocatif
            $this->Decl[$i + 2]->Des = '';        // Accusatif
            $this->Decl[$i + 3]->Des = 'is';      // Génitif
            $this->Decl[$i + 4]->Des = 'i';       // Datif
            $this->Decl[$i + 5]->Des = 'i';       // Ablatif
            }
        elseif ($d == '3er1')
            {
            $this->Decl[$i + 0]->Des = '';        // Nominatif
            $this->Decl[$i + 1]->Des = '';        // Vocatif
            $this->Decl[$i + 2]->Des = '';        // Accusatif
            $this->Decl[$i + 3]->Des = 'is';      // Génitif
            $this->Decl[$i + 4]->Des = 'i';       // Datif
            $this->Decl[$i + 5]->Des = 'e';       // Ablatif
            }
        elseif (in_array($d, Array('3as', '3es1', '3is', '3ns1', '3ns2', '3os')))
            {
            $this->Decl[$i + 0]->Rac = substr($Radic, 0, -1); // Nominatif
            $this->Decl[$i + 1]->Rac = substr($Radic, 0, -1); // Vocatif
            
            $this->Decl[$i + 0]->Des = 's';       // Nominatif
            $this->Decl[$i + 1]->Des = 's';       // Vocatif
            $this->Decl[$i + 2]->Des = 'em';      // Accusatif
            $this->Decl[$i + 3]->Des = 'is';      // Génitif
            $this->Decl[$i + 4]->Des = 'i';       // Datif
            $this->Decl[$i + 5]->Des = 'e';       // Ablatif
            }
        elseif ($d == '3es2')
            {
            $this->Decl[$i + 0]->Rac = substr($Radic, 0, -2); // Nominatif
            $this->Decl[$i + 1]->Rac = substr($Radic, 0, -2); // Vocatif
            
            $this->Decl[$i + 0]->Des = 'es';       // Nominatif
            $this->Decl[$i + 1]->Des = 'es';       // Vocatif
            $this->Decl[$i + 2]->Des = 'em';      // Accusatif
            $this->Decl[$i + 3]->Des = 'is';      // Génitif
            $this->Decl[$i + 4]->Des = 'i';       // Datif
            $this->Decl[$i + 5]->Des = 'e';       // Ablatif
            }
        elseif ($d == '3isg')
            {
            $this->Decl[$i + 0]->Rac = substr($Radic, 0, -1); // Nominatif
            $this->Decl[$i + 1]->Rac = substr($Radic, 0, -1); // Vocatif
            $this->Decl[$i + 2]->Rac = substr($Radic, 0, -1); // Accusatif
            
            $this->Decl[$i + 0]->Des = 's';       // Nominatif
            $this->Decl[$i + 1]->Des = 's';       // Vocatif
            $this->Decl[$i + 2]->Des = 'n/m';     // Accusatif
            $this->Decl[$i + 3]->Des = 'is';      // Génitif
            $this->Decl[$i + 4]->Des = 'i';       // Datif
            $this->Decl[$i + 5]->Des = 'e';       // Ablatif
            }
        elseif (in_array($d, Array('3us1', '3us2')))
            {
            $this->Decl[$i + 0]->Rac = substr($Radic, 0, -2); // Nominatif
            $this->Decl[$i + 1]->Rac = substr($Radic, 0, -2); // Vocatif
            $this->Decl[$i + 2]->Rac = substr($Radic, 0, -2); // Accusatif
            
            $this->Decl[$i + 0]->Des = 'us';      // Nominatif
            $this->Decl[$i + 1]->Des = 'us';      // Vocatif
            $this->Decl[$i + 2]->Des = 'us';      // Accusatif
            $this->Decl[$i + 3]->Des = 'is';      // Génitif
            $this->Decl[$i + 4]->Des = 'i';       // Datif
            $this->Decl[$i + 5]->Des = 'e';       // Ablatif
            }
        elseif (in_array($d, Array('3ax', '3ix')))
            {
            $this->Decl[$i + 0]->Rac = substr($Radic, 0, -1); // Nominatif
            $this->Decl[$i + 1]->Rac = substr($Radic, 0, -1); // Vocatif
            
            $this->Decl[$i + 0]->Des = 'x';       // Nominatif
            $this->Decl[$i + 1]->Des = 'x';       // Vocatif
            $this->Decl[$i + 2]->Des = 'em';      // Accusatif
            $this->Decl[$i + 3]->Des = 'is';      // Génitif
            $this->Decl[$i + 4]->Des = 'i';       // Datif
            $this->Decl[$i + 5]->Des = 'e';       // Ablatif
            }
        elseif ($d == '3ex')
            {
            $this->Decl[$i + 0]->Rac = substr($Radic, 0, -2); // Nominatif
            $this->Decl[$i + 1]->Rac = substr($Radic, 0, -2); // Vocatif
            
            $this->Decl[$i + 0]->Des = 'ex';      // Nominatif
            $this->Decl[$i + 1]->Des = 'ex';      // Vocatif
            $this->Decl[$i + 2]->Des = 'em';      // Accusatif
            $this->Decl[$i + 3]->Des = 'is';      // Génitif
            $this->Decl[$i + 4]->Des = 'i';       // Datif
            $this->Decl[$i + 5]->Des = 'e';       // Ablatif
            }
        elseif ($d == '3s')
            {
            $this->Decl[$i + 0]->Rac = substr($Radic, 0, -1); // Nominatif
            $this->Decl[$i + 1]->Rac = substr($Radic, 0, -1); // Vocatif
            
            $this->Decl[$i + 0]->Des = 's';       // Nominatif
            $this->Decl[$i + 1]->Des = 's';       // Vocatif
            $this->Decl[$i + 2]->Des = 'em';      // Accusatif
            $this->Decl[$i + 3]->Des = 'is';      // Génitif
            $this->Decl[$i + 4]->Des = 'i';       // Datif
            $this->Decl[$i + 5]->Des = 'e';       // Ablatif
            }
            
        // --- 4ème déclinaison
        
        elseif (in_array($d, Array('4_mf', '4a')))
            {
            $this->Decl[$i + 0]->Des = 'us';      // Nominatif
            $this->Decl[$i + 1]->Des = 'us';      // Vocatif
            $this->Decl[$i + 2]->Des = 'um';      // Accusatif
            $this->Decl[$i + 3]->Des = 'us';      // Génitif
            $this->Decl[$i + 4]->Des = 'ui';      // Datif
            $this->Decl[$i + 5]->Des = 'u';       // Ablatif
            }
        elseif ($d == '4_n')
            {
            $this->Decl[$i + 0]->Des = 'u';       // Nominatif
            $this->Decl[$i + 1]->Des = 'u';       // Vocatif
            $this->Decl[$i + 2]->Des = 'u';       // Accusatif
            $this->Decl[$i + 3]->Des = 'us';      // Génitif
            $this->Decl[$i + 4]->Des = 'ui';      // Datif
            $this->Decl[$i + 5]->Des = 'u';       // Ablatif
            }
        
        // --- 5ème déclinaison
        
        elseif ($d == '5')
            {
            $this->Decl[$i + 0]->Des = 'es';       // Nominatif
            $this->Decl[$i + 1]->Des = 'es';       // Vocatif
            $this->Decl[$i + 2]->Des = 'em';       // Accusatif
            $this->Decl[$i + 3]->Des = 'ei';       // Génitif
            $this->Decl[$i + 4]->Des = 'ei';       // Datif
            $this->Decl[$i + 5]->Des = 'e';        // Ablatif
            }
            
        // ---
        
        else
            {
            $this->isDeclined = False;
            }
        
        return;

    
    
        }



/***************************************************************************//**
 * Extraction du radical du parfait/supin d'un verbe.                    
 * Le retour se fait en compléta,t les attributs correspondants de       
 * l'objet.                                                              
 *                                                                       
 * @param string $item Mot brut complet                                  
 *                                                                       
 * @return none                                                          
 */
    function ExtraireRadicParfSupin($item)
        {
        
        
        $n = 0;
        $p1 = 0;
        $p2 = 0;
        $ParfAbr = '';
        $SupAbr ='';
        // --------
        
        // Principe de l'extraction :
        //    faire concorder la première lettre du radical abrégé
        //    avec sa dernière occurrence dans le radical du présent
        
        
        // --- cas particulier : 1ère déclinaison ---
        if ($this->curNumConj == '1')
            {
            $this->Vradic2 = $this->Vradic1 . 'av';
            $this->Vradic3 = $this->Vradic1 . 'at';
                
            if (strpos($item, '/-/') !== False)                 // parfait défectif
                {
                $this->Vradic2 = '-';
                }
                
            if ((strpos($item, '/- ') !== False) or (strpos($item, '/-[') !== False))
                {
                $this->Vradic3 = '-';
                }
                
            return ;
            }
        
        
        // --- extraire le radical abrégé du parfait
        
        for ($n = 0; $n <= 1; $n++)   // (2 itérations)
            {  
            $p1 = strpos($item, '/', $p1 + 1);  // cherche le 2ème '/'
            }
        
        $p2 = strpos($item, '/', $p1 + 1);
        
        $ParfAbr = substr($item, $p1 + 1, $p2 - $p1 -2); 	// nb: ote le i 'final'
        
        
        if ($ParfAbr == '')                   // parfait défectif
            {
            $ParfAbr = '-';
            }
        
        // --- extraire le radical abrégé du supin
        
        $p1 = $p2;
        $p2 = strpos($item, 'um', $p1);
        
        if ($p2 > 0)
            {
            $SupAbr = substr($item, $p1 + 1, $p2 - $p1 -1);
            }
        else
            {
            $SupAbr = '-';                     // supin défectif
            }
        
        
        // --- coller pour reconstituer le radical du parfait
        
        $p1 = -1;
        $p2 = 0;
        
        while (1)
            {
            $p1 = strpos($this->Vradic1, $ParfAbr[0], $p1 + 1);
            
            if ($p1 === False)
                {
                break;
                }
            $p2 = $p1;
            }
        
        if ($p2 > 0)
            {
            $this->Vradic2 = substr($this->Vradic1, 0, $p2) . $ParfAbr;
            }
        else
            {
            $this->Vradic2 = $ParfAbr;
            }
        
        // --- coller pour reconstituer le radical du supin
        
        $p1 = -1;
        $p2 = 0;
        
        while (1)
            {
            $p1 = strpos($this->Vradic1, $SupAbr[0], $p1 + 1);
            if ($p1 === False)
                {
                break;
                }
            $p2 = $p1;
            }
        
        if ($p2 > 0)
            {
            $this->Vradic3 = substr($this->Vradic1, 0, $p2)  . $SupAbr;
            }
        else
            {
            $this->Vradic3 = $SupAbr;
            }
        
        }




/***************************************************************************//**
 * Analyse du code grammatical pour définir le type de déclinaison/conjugaison
 * à appliquer puis les méthodes de déclinaison/conjugaison sont appelées et la
 * liste correspondante est complétée.
 * 
 * Cette méthode est invoquée en fin de constructeur
 *
 * @param string $item Mot brut complet
 */
    function __definirDeclinaison($item) //(Private)
        {
        
        $p1 = 0;
        $p2 = 0;
        $idGram = Array();
        $Radic = '';
        
        
        // ===========================================================
        // === Initialiser les données de sortie de la classe         
        // ===========================================================
        // Decl : Liste des résultats de déclinaison (indices 0-35) 
        // Conj : Liste des résultats de conjuguaison (indices 0-11)
        // 2 listes d'éléments vides au départ                        
        
        $i = 0;
        for ($i = 0; $i <= 35; $i++) {$this->Decl[$i] = new MotCas();}
        for ($i = 0; $i <= 11; $i++) {$this->Conj[$i] = new MotCas();}
        
        
        
        // =====================================================
        
        
        $item = trim($item);
        
        
        // === Reconnaître une entrée hors-contexte à ne pas traiter
        //     (les items commençant par '#' ne sont pas traités, simples commentaires dans le dico)
        if ($item == '' or $item[1] == '#')
            {
            $this->isRem = True;
            $this->isDeclNone = True; // indéclinable
            return;
            }
        
        
        // ==============================================================================
        // === Extraire la chaine d'identification grammaticale (contenue entre crochets)
        // ==============================================================================
        
        $p1 = strpos($item, '[');
        $p2 = strpos($item, ']');
        
        
        if ($p1 > 0)      // une chaîne entre crochets est présente
            {
            if ($p2 < $p1) 
                {
                $p2 = strlen($item);
                } 
            
            // convertir en Liste
            $idGram = trim(substr($item, $p1 + 1, $p2 - ($p1 + 1)));
            $idGram = explode(' ', $idGram);
            }
        else // absence de chaîne entre crochets
            {
            $this->isDeclUnknown = True; // déclinable ou non ???
            
            // >> tenter d'attribuer des données grammaticales et relancer la procédure si possible
            $this->AnalyseAutomatique($item);
            return ;
            }
        
        // ================================================================
        // === Définir le type grammatical (parsing du code entre crochet) 
        // ================================================================
        
        // Valeurs si aucun motif n'a été reconnu dans le code grammatical
        $this->isUnknownStatus = True;
        $this->GramInfo = '(Statut grammatical non précisable)';
        

        // --- Scan des Items instaurant une chaîne grammaticale (GramInfo)  
        
        foreach ($idGram as $it)
            {
            if ($it == 'adj')
                {
                $this->isAdjective = True;
                $this->GramInfo = 'Adjectif';
                }
            elseif ($it == 's')
                {
                $this->isSubstantive = True;
                $this->GramInfo = 'Substantif';
                }
            elseif ($it == 'adv')
                {
                $this->isAdverb = True;
                $this->isDeclNone = True;
                $this->GramInfo = 'Adverbe';
                }
            elseif (substr($it, 0, 3) == 'suf') // --- Suffixes
                {
                $this->isSuffix = True;
                
                if (($it == 'suf-LG') or ($it == 'suf-GL'))
                    {
                    $this->isFromGreek = True;
                    $this->isFromLatin = True;
                    $this->GramInfo = 'Suffixe ou racine suffixée (dans les composés à racines latines ou grecques)';
                    }
                elseif ($it == 'suf-L')
                    {
                    $this->isFromLatin = True;
                    $this->GramInfo = 'Suffixe ou racine suffixée (dans les composés à racines latines)';
                    }
                elseif ($it == 'suf-G')
                    {
                    $this->isFromGreek = True;
                    $this->GramInfo = 'Suffixe ou racine suffixée (dans les composés à racines grecques)';
                    }
                elseif ($it == 'suf')
                    {
                    $this->GramInfo = 'Suffixe ou racine suffixée';
                    }
                }
            elseif (substr($it, 0, 3) == 'prf') // --- Préfixes
                {
                $this->isPrefix = True;
                $this->isDeclNone = True;
                
                if (($it == 'prf-LG') or ($it == 'prf-GL'))
                    {
                    $this->isFromGreek = True;
                    $this->isFromLatin = True;
                    $this->GramInfo = 'Préfixe ou racine préfixée (dans les composés à racines latines ou grecques)';
                    }
                elseif ($it == 'prf-L')
                    {
                    $this->isFromLatin = True;
                    $this->GramInfo = 'Préfixe ou racine préfixée (dans les composés à racines latines)';
                    }
                elseif ($it == 'prf-G')
                    {
                    $this->isFromGreek = True;
                    $this->GramInfo = 'Préfixe ou racine préfixée (dans les composés à racines grecques)';
                    }
                elseif ($it == 'prf')
                    {
                    $this->GramInfo = 'Préfixe ou racine préfixée';
                    }
                }
            elseif ($it == 's-adj')
                {
                $this->isSubstantive = True;
                $this->isAdjective = True;
                $this->GramInfo = 'Substantif utilisé adjectivement';
                }
            elseif (substr($it, 0, 3) == 'cmp') // --- Comparatifs
                {
                $this->isComparative = True;
                
                if ($it == 'cmp')
                    {
                    $this->GramInfo = 'Adjectif comparatif de supériorité';
                    }
                elseif ($it == 'cmp-i')
                    {
                    $this->GramInfo = 'Adjectif comparatif de supériorité (déclinaison spéciale)';
                    }
                }
            elseif ($it == 'sup')
                {
                $this->isSuperlative = True;
                $this->GramInfo = 'Adjectif superlatif';
                }
            elseif ($it == 'conj')
                {
                $this->isConjunction = True;
                $this->isDeclNone = True;
                $this->GramInfo = 'Conjonction';
                }
            elseif ($it == 'dem')
                {
                $this->isDemonstrative = True;
                $this->GramInfo = 'Pronom-adjectif démonstratif';
                }
            elseif (substr($it, 0, 4) == 'prep') // --- Prépositions
                {
                $this->isPreposition = True;
                
                if ($it == 'prep+ab')
                    {
                    $this->isWithAblative = True;
                    $this->isDeclNone = True;
                    $this->GramInfo = 'Préposition (+ ablatif)';
                    }
                elseif ($it == 'prep+ac')
                    {
                    $this->isWithAccusative = True;
                    $this->isDeclNone = True;
                    $this->GramInfo = 'Préposition (+ accusatif)';
                    }
                elseif ($it == 'prep+ac/ab' or $it == 'prep+ab/ac')
                    {
                    $this->isWithAccusative = True;
                    $this->isWithAblative = True;
                    $this->isDeclNone = True;
                    $this->GramInfo = 'Préposition (+ accusatif ou ablatif)';
                    }
                elseif ($it == 'prep')
                    {
                    $this->isDeclNone = True;
                    $this->GramInfo = 'Préposition';
                    }
                }
            elseif ($it[0] == 'v')
                {
                $this->isVerb = True;
                $this->isDeclNone = True;
                $this->GramInfo = 'Verbe';
                }
            elseif (substr($it, 0, 3) == 'num')
                {
                $this->isNumeral = True;
                $this->GramInfo = 'Expression numérique';
                }
            elseif ($it == 'symb')
                {
                $this->isSymbol = True;
                $this->isDeclNone = True;
                $this->GramInfo = '(Symbole)';
                }
            elseif ($it == 'loc')
                {
                $this->isLocution = True;
                $this->toNotDecl = True;  // ne pas décliner;
                $this->GramInfo = 'Locution ou formule consacrée';
                }
            elseif ($it == 'rel')
                {
                $this->isRelative = True;
                $this->GramInfo = 'Pronom-adjectif relatif';   
                }
            
            }
            
        // --- Scan des Items complétant la chaîne grammaticale
        
        foreach ($idGram as $it)
            {
            if ($it[0] == '1')
                {
                $this->isDecl1 = True;
                $this->GramInfo .= ' 1ère déclinaison';
                }
            elseif ($it[0] == '2')
                {
                $this->isDecl2 = True;
                $this->GramInfo .= ' 2ème déclinaison';
                }
            elseif ($it[0] == '3')
                {
                $this->isDecl3 = True;
                $this->GramInfo .= ' 3ème déclinaison';
                }
            elseif ($it[0] == '4')
                {
                $this->isDecl4 = True;
                $this->GramInfo .= ' 4ème déclinaison';
                }
            elseif ($it[0] == '5')
                {
                $this->isDecl5 = True;
                $this->GramInfo .= ' 5ème déclinaison';
                }
            elseif ($it[0] == 'A')
                {
                $this->isGroupA = True;
                $this->GramInfo .= ' 1er groupe';
                }
            elseif ($it[0] == 'B')
                {
                $this->isGroupB = True;
                $this->GramInfo .= ' 2ème groupe';
                }
            elseif ($it[0] == 'C')
                {
                $this->isGroupC = True;
                $this->GramInfo .= ' 2ème groupe (\'C\')';
                }
            }
        
        // --- Items du genre
        
        if ( (in_array('m', $idGram) and in_array('f', $idGram)) or in_array('fm', $idGram) or in_array('mf', $idGram) )
            {
            $this->isMasculine = True;
            $this->isFeminine = True;
            $this->GramInfo .= ' masculin et féminin';
            }
        elseif (in_array('m', $idGram))
            {
            $this->isMasculine = True;
            $this->GramInfo = $this->GramInfo . ' masculin';
            }
        elseif (in_array('f', $idGram))
            {
            $this->isFeminine = True;
            $this->GramInfo .= ' féminin';
            }
        elseif (in_array('n', $idGram))
            {
            $this->isNeutral = True;
            $this->GramInfo .= ' neutre';
            }
        
        // --- Items du Nombre
        
        if (in_array('pl', $idGram))          // utilisé (décliné) au pluriel seul
            {
            $this->isPlural = True;
            $this->GramInfo .= ' pluriel';
            }
        elseif (in_array('sg', $idGram))        // utilisé (décliné) au singulier seul
            {
            $this->isSingular = True;
            $this->GramInfo .= ' singulier';
            }
        else                       // par défaut : utilisé (décliné) au singulier et pluriel
            {
            $this->isSingular = True;
            $this->isPlural = True;
            }
        
        // --- Items cloturant la chaîne grammaticale
        
        if (in_array('abr', $idGram))
            {
            $this->isAbbreviation = True;
            $this->isDeclNone = True;
            $this->GramInfo .= '(abréviation)';
            }
        
        // === Forcer l'indéclinabilité réelle ===
        
        if (in_array('indecl', $idGram))
            {
            $this->isDeclNone = True;
            $this->GramInfo .= ' (expression indéclinable)';
            }
        
        
        // ================================================================
        // === Extraire le mot de base (= 1er mot avec sa désinence)       
        // ================================================================

        $this->__MotBase = $item;
        
        $n = strpos($item, '/');
        if ($n > 0)
            {
            $this->__MotBase = substr($item, 0, $n);
            }
        else
            {
            $n = strpos($item, ' ');
            if ($n > 0)
                {
                $this->__MotBase = substr($item, 0, $n);
                }
            else
                {
                $n = strpos($item, '[');
                if ($n > 0)
                    {
                    $this->__MotBase = substr($item, 0, $n);
                    }
                else
                    {
                    $n = strpos($item, '(');
                    if ($n > 0)
                        {
                        $this->__MotBase = substr($item, 0, $n);
                        }
                    }
                }
            }
            
        
        $this->__MotBase = trim($this->__MotBase);
        
        
        // Formatage d'affichage du mot de base des suffixes    
        if ($this->isSuffix)
            $this->__MotBase = '...-' . $this->__MotBase;
        
        
        
        
        // ===========================================================
        // === Traitements préalables et aiguillage vers              
        // === les modules de déclinaison/conjugaisons                
        // === en fonction du type grammatical                        
        // ===========================================================
        
        
        
        // --------------------------------------------------
        if ($this->isDemonstrative) // --- Démonstratifs        
        // --------------------------------------------------
            {
            $this->DeclinerDemonstratif($this->__MotBase);
        
            }
        // --------------------------------------------------
        elseif ($this->isVerb) // --- Verbe                       
        // --------------------------------------------------
            {
            // --- Transitivité
            if (in_array('t', $idGram))
                {
                $this->isTransitive = True;
                $this->isDeclNone = True;
                $this->GramInfo = $this->GramInfo . ' transitif';
                }
            elseif (in_array('i', $idGram))
                {
                $this->isIntransitive = True;
                $this->isDeclNone = True;
                $this->GramInfo = $this->GramInfo . ' intransitif';
                }
            elseif (in_array('ti', $idGram))
                {
                $this->isTransitive = True;
                $this->isIntransitive = True;
                $this->isDeclNone = True;
                $this->GramInfo = $this->GramInfo . ' transitif et intransitif';
                }
                
            // --- Définir conjugaison, extraire radical du présent et infininif
            //    puis conjuguer par défaut au présent indicatif actif)
            
            if (in_array('v1', $idGram))
                {
                $this->isConj1 = True;
                $this->curNumConj = '1';
                $this->Vradic1 = substr($this->__MotBase, 0, -1);             // radical du présent
                $this->Vinfinitif = $this->Vradic1 . 'are';
                $this->ExtraireRadicParfSupin($item);
                $this->ConjUpdate('Indicatif', 'Present');
                $this->GramInfo = $this->GramInfo . ' 1ère conjugaison';
                }
            elseif (in_array('v2', $idGram))
                {
                $this->isConj2 = True;
                $this->curNumConj = '2';
                $this->Vradic1 = substr($this->__MotBase, 0, -2);             // radical du présent
                $this->Vinfinitif = $this->Vradic1 . 'ere';
                $this->ExtraireRadicParfSupin($item);
                $this->ConjUpdate('indicatif', 'present');
                $this->GramInfo = $this->GramInfo . ' 2ème conjugaison';
                }
            elseif (in_array('v3', $idGram))
                {
                $this->isConj3 = True;
                $this->curNumConj = '3';
                $this->Vradic1 = substr($this->__MotBase, 0, -1);             // radical du présent
                $this->Vinfinitif = $this->Vradic1 . 'ere';
                $this->ExtraireRadicParfSupin($item);
                $this->ConjUpdate('indicatif', 'present');
                $this->GramInfo = $this->GramInfo . ' 3ème conjugaison';
                }
            elseif (in_array('v4', $idGram))
                {
                $this->isConj4 = True;
                $this->curNumConj = '4';
                $this->Vradic1 = substr($this->__MotBase, 0, -2);             // radical du présent
                $this->Vinfinitif = $this->Vradic1 . 'ire';
                $this->ExtraireRadicParfSupin($item);
                $this->ConjUpdate('indicatif', 'present');
                $this->GramInfo = $this->GramInfo . ' 4ème conjugaison';
                }
            elseif (in_array('vm', $idGram))
                {
                $this->isConjM = True;
                $this->curNumConj = 'm';
                $this->Vradic1 = substr($this->__MotBase, 0, -2);             // radical du présent
                $this->Vinfinitif = $this->Vradic1 . 'ere';
                $this->ExtraireRadicParfSupin($item);
                $this->ConjUpdate('indicatif', 'present');
                $this->GramInfo = $this->GramInfo . ' conjugaison mixte';
                }
            elseif (in_array('vs', $idGram))       // --- sum et dérivés
                {
                $this->isConjSum = True;
                $this->curNumConj = 's';
                $this->Vradic1 = substr($this->__MotBase, 0, -3);             // radical du présent
                $this->Vradic2 = substr($this->__MotBase, 0, -3) . 'fu';      // radical du parfait
                $this->Vradic3 = '-';                              // radical du supin
                $this->Vinfinitif = $this->Vradic1 . 'esse';
                $this->ConjUpdate('indicatif', 'present');
                $this->GramInfo = $this->GramInfo . ' (\'esse\' et dérivés : conjugaison irrégulière)';
                }
            elseif (in_array('vf', $idGram))       // --- fero et dérivés
                {
                $this->isConjFero = True;
                $this->curNumConj = 'f';
                $this->Vradic1 = substr($this->__MotBase, 0, -1);             // radical du présent
                $this->ExtraireRadicParfSupin($item);
                $this->Vinfinitif = $this->Vradic1 . 're';
                $this->ConjUpdate('indicatif', 'present');
                $this->GramInfo = $this->GramInfo . ' (\'ferre\' et dérivés : conjugaison irrégulière)';
                }
            
            
            } 
        
        
        // -----------------------------------------------------------------------------------------
        //elseif $this->isDeclNone or $this->toNotDecl: // --- réellement indéclinable ou à ne pas décliner 
        // -----------------------------------------------------------------------------------------
        //   pass;
        
        
        
        // -------------------------------------------------------------
        if ($this->isDecl1) // --- 1ère déclinaison                        
        // -------------------------------------------------------------
            {
            // --- racine
            if (substr($this->__MotBase, -2) == 'ae')       // mots pluriels écrits sans ligature
                {
                $Radic = substr($this->__MotBase, 0, -2);
                }
            elseif (substr($this->__MotBase, -2) == 'æ')    // mots pluriels écrits avec ligature (utf-8 donc 2 caractères)
                {
                $Radic = substr($this->__MotBase, 0, -2);
                }
            else
                {
                $Radic = substr($this->__MotBase, 0, -1);
                }
            
            // --- code de déclinaison
            $arg = '';
            if (in_array('1e', $idGram))
                {
                $arg = '1e';
                }
            elseif (in_array('1ae', $idGram))
                {
                $arg = '1ae';
                }
            elseif (in_array('1-', $idGram))
                {
                $this->isDeclUnknown = True;
                }
            else
                {
                $arg = '1';
                }
            
            // ---
            $this->DeclinerSubstantif($Radic, $arg);
        
            }
        // ----------------------------------------------------------------
        elseif ($this->isDecl2) // --- 2ème déclinaison                         
        // ----------------------------------------------------------------
            {
            $arg = '';
            if (in_array('2r1', $idGram))
                {
                $arg = '2r1';
                $Radic = substr($this->__MotBase, 0, -2) . 'r';
                }
            elseif (in_array('2r2', $idGram))
                {
                $arg = '2r2';
                $Radic = $this->__MotBase;
                }
            elseif (in_array('2on', $idGram))
                {
                $arg = '2on';
                $Radic = substr($this->__MotBase, 0, -2);
                }
            elseif (in_array('2-', $idGram))
                {
                $this->isDeclUnknown = True;
                }
            else
                {
                if ($this->isPlural and !$this->isSingular)       // mot au pluriel seul
                    {
                    $Radic = substr($this->__MotBase, 0, -1);
                    }
                else
                    {
                    $Radic = substr($this->__MotBase, 0, -2);
                    }
                if ($this->isNeutral)
                    {
                    $arg = '2_n';
                    }
                else
                    {
                    $arg = '2_mf';
                    }
                }
                
            // ---
            $this->DeclinerSubstantif($Radic, $arg);
        
            }
        // -----------------------------------------------------------------
        elseif ($this->isDecl3) // --- 3ème déclinaison                          
        // -----------------------------------------------------------------
            {
            $arg = '';
            if (in_array('3p', $idGram))          // parisyllabiques standards
                {
                if ($this->isNeutral)
                    {
                    $arg = '3p_n';
                    if (!$this->isSingular)
                        {
                        $Radic = substr($this->__MotBase, 0, -2);
                        }
                    else
                        {
                        $Radic = substr($this->__MotBase, 0,-1);
                        }
                    }
                else
                    {
                    $arg = '3p_mf';
                    $Radic = substr($this->__MotBase, 0, -2);
                    }
                }
            elseif (in_array('3pg', $idGram))       // parisyllabiques à déclinaison grécisante
                {
                $arg = '3pg';
                $Radic = substr($this->__MotBase, 0, -2);
                }
            elseif (in_array('3en', $idGram))
                {
                $arg = '3en';
                $Radic = substr($this->__MotBase, 0, -2) . 'in';
                }
            elseif (in_array('3on', $idGram) or in_array('3or', $idGram))
                {
                $arg = '3on';
                $Radic = $this->__MotBase;
                }
            elseif (in_array('3ar', $idGram))
                {
                $arg = '3ar';
                $Radic = $this->__MotBase;
                }
            elseif (in_array('3er1', $idGram))
                {
                $arg = '3er1';
                $Radic = $this->__MotBase;
                }
            elseif (in_array('3er2', $idGram))
                {
                $arg = '3er2';
                $Radic = $this->__MotBase;
                }
            elseif (in_array('3or', $idGram))
                {
                $arg = '3or';
                $Radic = $this->__MotBase;
                }
            elseif (in_array('3o1', $idGram))
                {
                $arg = '3o1';
                $Radic = $this->__MotBase . 'n';
                }
            elseif (in_array('3o2', $idGram))
                {
                $arg = '3o2';
                $Radic = substr($this->__MotBase, 0, -1) . 'in';
                }
            elseif (in_array('3as', $idGram))
                {
                $arg = '3as';
                $Radic = substr($this->__MotBase, 0,-1) . 't';
                }
            elseif (in_array('3es1', $idGram))
                {
                $arg = '3es1';
                $Radic = substr($this->__MotBase, 0, -1) . 'd';
                }
            elseif (in_array('3es2', $idGram))
                {
                $arg = '3es2';
                $Radic = substr($this->__MotBase, 0, -2) . 'it';
                }
            elseif (in_array('3is', $idGram))
                {
                $arg = '3is';
                $Radic = substr($this->__MotBase, 0, -1) . 'd';
                }
            elseif (in_array('3isg', $idGram))
                {
                $arg = '3isg';
                $Radic = substr($this->__MotBase, 0, -1) . 'd';
                }
            elseif (in_array('3ns1', $idGram))
                {
                $arg = '3ns1';
                $Radic = substr($this->__MotBase, 0, -1) . 't';
                }
            elseif (in_array('3ns2', $idGram))
                {
                $arg = '3ns2';
                $Radic = substr($this->__MotBase, 0, -1) . 'd';
                }
            elseif (in_array('3os', $idGram))
                {
                $arg = '3os';
                $Radic = substr($this->__MotBase, 0, -1) . 'r';
                }
            elseif (in_array('3us1', $idGram))
                {
                $arg = '3us1';
                $Radic = substr($this->__MotBase, 0, -2) . 'er';
                }
            elseif (in_array('3us2', $idGram))
                {
                $arg = '3us2';
                $Radic = substr($this->__MotBase, 0, -2) . 'or';
                }
            elseif (in_array('3ax', $idGram))
                {
                $arg = '3ax';
                $Radic = substr($this->__MotBase, 0, -1) . 'c';
                }
            elseif (in_array('3ix', $idGram))
                {
                $arg = '3ix';
                $Radic = substr($this->__MotBase, 0, -1) . 'c';
                }
            elseif (in_array('3ex', $idGram))
                {
                $arg = '3ex';
                $Radic = substr($this->__MotBase, 0, -2) . 'ic';
                }
            elseif (in_array('3a', $idGram))
                {
                $arg = '3a';
                $Radic = $this->__MotBase . 't';
                }
            elseif (in_array('3s', $idGram))
                {
                $arg = '3s';
                $Radic = substr($this->__MotBase, 0, -1) . 't';
                }
            elseif (in_array('3-', $idGram))
                {
                $this->isDeclUnknown = True;
                }
                
            // ---
            $this->DeclinerSubstantif($Radic, $arg);
        
            }
        // -------------------------------------------------------------------
        elseif ($this->isDecl4) // --- 4ème déclinaison                            
        // -------------------------------------------------------------------
            {
            // --- racine
            if (substr($this->__MotBase, -2) == 'us')
                {
                $Radic = substr($this->__MotBase, 0, -2);
                }
            else
                {
                $Radic = substr($this->__MotBase, 0, -1);
                }
            
            // --- décliner
              
            $arg = '';
            if (in_array('4a', $idGram))
                {
                $arg = '4a';
                }
            else
                {
                if ($this->isNeutral)
                  {
                  $arg = '4_n';
                  }
                else
                  {
                  $arg = '4_mf';
                  }
                }
            
            // ---
            $this->DeclinerSubstantif($Radic, $arg);
        
            }
        // -------------------------------------------------------------------------
        elseif ($this->isDecl5) // --- 5ème déclinaison                                  
        // -------------------------------------------------------------------------
            {
            $Radic = substr($this->__MotBase, 0, -2);
            $arg = '5';
            
            // ---
            $this->DeclinerSubstantif($Radic, $arg);
        
            }
        // ------------------------------------------------------------------------
        elseif ($this->isGroupA) // --- Groupe A                                        
        // ------------------------------------------------------------------------
            {
            
            $arg = '';
            if (in_array('Aa', $idGram))
                {
                $arg = 'Aa';
                $Radic = $this->__MotBase;
                }
            elseif (in_array('Ab', $idGram))
                {
                $arg = 'Ab';
                $Radic = substr($this->__MotBase, 0, -2) . 'r';
                }
            elseif (in_array('A-', $idGram))
                {
                $this->isDeclUnknown = True;
                }
            elseif (in_array('A', $idGram))
                {
                $arg = 'A';
                
                
                if ((substr($this->__MotBase, -1) == 'æ') or (substr($this->__MotBase, -1) == 'i'))
                    {
                    $Radic = substr($this->__MotBase, 0, -1);
                    }
                else
                    {
                    $Radic = substr($this->__MotBase, 0, -2);
                    }
                }
                
            // ---
            $this->DeclinerAdjectif($Radic, $arg);
        
            }
        // --------------------------------------------------------------
        elseif ($this->isGroupB) // --- Groupe B                              
        // --------------------------------------------------------------
            {
            $arg = '';
            if (in_array('Bp', $idGram))
                {
                $arg = 'Bp';
                $Radic = substr($this->__MotBase, 0, -2);
                }
            elseif (in_array('Bns', $idGram))
                {
                $arg = 'Bns';
                $Radic = substr($this->__MotBase, 0, -1) . 't';
                }
            elseif (in_array('Bex', $idGram))
                {
                $arg = 'Bex';
                $Radic = substr($this->__MotBase, 0, -2) . 'ic';
                }
            elseif (in_array('Ber', $idGram))
                {
                $arg = 'Ber';
                $Radic = substr($this->__MotBase, 0, -2) . 'r';
                }
            elseif (in_array('Bx', $idGram))
                {
                $arg = 'Bx';
                $Radic = substr($this->__MotBase, 0, -1) . 'c';
                }
            elseif (in_array('Bor', $idGram))
                {
                $arg = 'Bor';
                $Radic = $this->__MotBase;
                }
            elseif (in_array('Bus', $idGram))
                {
                $arg = 'Bus';
                $Radic = substr($this->__MotBase, 0, -2) . 'er';
                }
            elseif (in_array('B-', $idGram))
                {
                $this->isDeclUnknown = True;
                }
            else
                {
                $this->isDeclUnknown = True;
                }
            
            // ---
            $this->DeclinerAdjectif($Radic, $arg);
        
            }
        // -------------------------------------------------------------------------
        elseif ($this->isGroupC) // --- Groupe C                                         
        // -------------------------------------------------------------------------
            {
            $arg = '';
            if (in_array('Ci', $idGram))
                {
                $arg = 'Ci';
                $Radic = substr($this->__MotBase, 0, -2);
                }
            elseif (in_array('Co', $idGram))
                {
                $arg = 'Co';
                $Radic = substr($this->__MotBase, 0, -2);
                }
            elseif (in_array('C-', $idGram))
                {
                $this->isDeclUnknown = True;
                }
            else
                {
                $this->isDeclUnknown = True;
                }
                
            // ---
            $this->DeclinerAdjectif($Radic, $arg);
            
            }
        // -------------------------------------------------------------------------
        elseif ($this->isComparative) // --- Comparatif                                  
        // -------------------------------------------------------------------------
            {
            $arg = '';
            if (in_array('cmp-i', $idGram))
                {
                $arg = 'cmp-i';
                $Radic = substr($this->__MotBase, 0, -2);
                }
            else
                {
                $arg = 'cmp';
                $Radic = substr($this->__MotBase, 0, -2);
                }
                
            // ---
            $this->DeclinerAdjectif($Radic, $arg);
            
            }
        // -------------------------------------------------------------------------
        elseif ($this->isNumeral) // ---- Numéral                                        
        // -------------------------------------------------------------------------
            {
            $arg = '';
            if (in_array('num1', $idGram))
                {
                $arg = 'num1';
                $Radic = substr($this->__MotBase, 0, -2);
                $this->GramInfo .= ' (basé sur \'unus\')';
                }
            elseif (in_array('num2', $idGram))
                {
                $arg = 'num2';
                $Radic = substr($this->__MotBase, 0, -1);
                $this->GramInfo .= ' (basé sur \'duo\')';
                }
            elseif (in_array('num3', $idGram))
                {
                $arg = 'num3';
                $Radic = substr($this->__MotBase, 0, -2);
                $this->GramInfo .= ' (basé sur \'tres\')';
                }
            elseif ($this->isDeclNone)
                {
                // pass;
                }
            else
                {
                $this->isDeclUnknown = True;
                }
                
            $this->DeclinerNumeral($Radic, $arg);
            
            }
        // -------------------------------------------------------------------------
        elseif ($this->isRelative) // ---- Pronom relatif                                
        // -------------------------------------------------------------------------
            {
            $this->declinerRelatif();
            }
        // ---------------------------------------------------------------------------
        else // --- Aucune déclinaison n'a été trouvée                               
        // ---------------------------------------------------------------------------
            {
            $this->isDeclUnknown = True;
            }
            
        return;

    
    
        }

    }





class MotCas
    {
    //Structure des données déclinées
    
    var $Rac = '';       // racine
    var $Des = '';       // désinence

    }


class Widget
    {
    
    var $oLatin = '';
    
/***************************************************************************//**
 * Affiche les données de verbe de l'objet dans les cases ad hoc
 *
 *  
 */
    function AfficherVerbe()
        {
        
        $tBox = '';
        $t = '';
        
        
        // ------------------------------------------
        // --- Tableau des radicaux du verbe         
        // ------------------------------------------
        
        echo '<table><tr><td  class="decl"><table><tr><td>Infinitif </td><td>: ', $this->oLatin->Vinfinitif, '</td></tr>
        <tr><td>Radical du Présent </td><td>: ', $this->oLatin->Vradic1 . '-', '</td></tr>
        <tr><td>Radical du Parfait </td><td>: ', $this->oLatin->Vradic2 . '-', '</td></tr>
        <tr><td>Radical du Supin </td><td>: ', $this->oLatin->Vradic3 . '-', '</td></tr>
        </table></td></tr></table>';
        
        
        // ------------------------------------
        // --- Affichage de la conjugaison     
        // ------------------------------------
        
        foreach (Array('INDICATIF', 'SUBJONCTIF/CONDITIONNEL') as $mode)
            {
            foreach (Array('PRESENT', 'IMPARFAIT', 'FUTUR', 'PARFAIT', 'PLUS-QUE-PARFAIT', 'FUTUR ANTERIEUR') as $temps)
                {
                
                // --- Etiquette du Mode et du Temps
                echo '<h2><span class="ttr">', $mode, ' ', $temps, '</span></h2>';
                
                // --- Exemples
                
                $t = 'Exemple basé sur \'aimer\' : ';
        
                if ($mode == 'INDICATIF')
                    {
                    if ($temps == 'PRESENT')
                        {
                        $t .= '(Actif) <i>J\'aime. </i>(Passif)<i> Je suis aimé</i>';
                        }
                    elseif ($temps == 'IMPARFAIT')
                        {
                        $t .= '(Actif) <i>J\'aimais. </i>(Passif)<i> J\'étais aimé</i>';
                        }
                    elseif ($temps == 'FUTUR')
                        {
                        $t .= '(Actif) <i>J\'aimerai. </i>(Passif)<i> Je serai aimé</i>';
                        }
                    elseif ($temps == 'PARFAIT')
                        {
                        $t .= '(Actif) <i>J\'ai aimé, j\'aimai. </i>(Passif)<i> J\'ai été aimé, je fus aimé</i>';
                        }
                    elseif ($temps == 'PLUS-QUE-PARFAIT')
                        {
                        $t .= '(Actif) <i>J\'avais aimé. </i>(Passif)<i> J\'avais été aimé</i>';
                        }
                    elseif ($temps == 'FUTUR ANTERIEUR')
                        {
                        $t .= '(Actif) <i>J\'aurai aimé. </i>(Passif)<i> J\'aurai été aimé</i>';
                        }
                    }
                elseif ($mode == 'SUBJONCTIF/CONDITIONNEL')
                    {
                    if ($temps == 'PRESENT')
                        {
                        $t .= '(Actif) <i>Que j\'aime / J\'aimerais. </i>(Passif)<i> Que je sois aimé / Je serais aimé</i>';
                        }
                    elseif ($temps == 'IMPARFAIT')
                        {
                        $t .= '(Actif) <i>Que j\'aimasse / J\'aimerais. </i>(Passif)<i> Que je fusse aimé / Je serais aimé</i>';
                        }
                    elseif ($temps == 'FUTUR')
                        {
                        $t = '<span class="warn">(ce temps n\'existe pas au subjonctif)</span>';
                        }
                    elseif ($temps == 'PARFAIT')
                        {
                        $t .= '(Actif) <i>Que j\'aie aimé. </i>(Passif)<i> Que j\'aie été aimé</i>';
                        }
                    elseif ($temps == 'PLUS-QUE-PARFAIT')
                        {
                        $t .= '(Actif) <i>Que j\'eusse aimé / J\'aurais aimé. </i>(Passif)<i> Que j\'eusse été aimé / J\'aurais été aimé</i>';
                        }
                    elseif ($temps == 'FUTUR ANTERIEUR')
                        {
                        $t = '<span class="warn">(ce temps n\'existe pas au subjonctif)</span>';
                        }
                    }
                
                echo '<p class="exem">', $t, '</p>';
                
                // --- Mise à jour du tableau de conjugaison
                $this->oLatin->ConjUpdate($mode, $temps);
                
                // --- Tableau de la conjugaison
                
                if (! ($mode == 'SUBJONCTIF/CONDITIONNEL' && ($temps == 'FUTUR' || $temps == 'FUTUR ANTERIEUR')))
                    {
                    echo '<table><tr><td  class="decl"><table><tr><td>&nbsp;</td><td class="gnr"><b>Actif</b></td><td class="gnr"><b>Passif</b></td></tr>';
                    
                    echo '<tr><td>1ère pers. sing.</td>';
                    $this->writeCase(0, $this->oLatin->Conj);
                    $this->writeCase(6, $this->oLatin->Conj);
                    echo '</tr>';
                    
                    echo '<tr><td>2ème pers. sing.</td>';
                    $this->writeCase(1, $this->oLatin->Conj);
                    $this->writeCase(7, $this->oLatin->Conj);
                    echo '</tr>';
                        
                    echo '<tr><td>3ème pers. sing.</td>';
                    $this->writeCase(2, $this->oLatin->Conj);
                    $this->writeCase(8, $this->oLatin->Conj);
                    echo '</tr>';
                    
                    echo '<tr><td>1ère pers. plur.</td>';
                    $this->writeCase(3, $this->oLatin->Conj);
                    $this->writeCase(9, $this->oLatin->Conj);
                    echo '</tr>';
                    
                    echo '<tr><td>2ème pers. plur.</td>';
                    $this->writeCase(4, $this->oLatin->Conj);
                    $this->writeCase(10, $this->oLatin->Conj);
                    echo '</tr>';
                        
                    echo '<tr><td>3ème pers. plur.</td>';
                    $this->writeCase(5, $this->oLatin->Conj);
                    $this->writeCase(11, $this->oLatin->Conj);
                    echo '</tr>';
                    
                    
                    echo '</table></td></tr></table>';
                    }
                }
            }
       
        
        
        return;
        
        
        
        
        }
    
    
    
    
    
/***************************************************************************//**
 * 1) Charge le mot brut dans un objet "Mot Latin Declinator"            
 * 2) Récupère les données fournies en retour par celui-ci dans ses      
 *    propriétés (attributs et listes)                                   
 * 3) Affiche les données de retour dans l'interface du widget           
 *                                                                       
 * @param string $motBrut Mot brut complet                               
 *                                                                       
 * @return none                                                          
 */
    function display($motBrut)
        {
        
        
        // -----------------------------------------------
        
        $i = 0;
        $tBox = '';
        
        // ------------------------------------------------
        // --- Instancier l'objet "Mot décliné/conjugué"   
        // ------------------------------------------------
        
        $this->oLatin = new cl_LatinWord($motBrut);
        
        // --- Afficher le mot brut
        
        // Supprimer à partir du premier crochet ouvrant
        $t = '<b>' . $motBrut;
        if(strpos($t, '['))
            {
            $t = substr($t, 0, strpos($t, '[')) . '</b>';
            }
        else
            {
            $t .= '</b>';
            }
        
        
        echo '<h1>', $t, '</h1>';
        
        // --- Afficher les infos grammaticales conviviales
        
        echo '<p class="gram">', $this->oLatin->GramInfo, '</p>';
        
        
        
        
        // ==========================================================================
        if ($this->oLatin->isDeclined)       // le mot a été effectivement décliné       
        // ==========================================================================
            {
            
            //(l'utilisation d'une cellule come conteneur est un contournement du bug de IE pour le padding des tables)
            echo '<table><tr><td  class="decl"><table>';
            
              
            // ----------------------------------------
            // --- Affichage des cases du Singulier    
            // ----------------------------------------
            
            echo '<tr><td><b>Singul.</b></td>';
            
            // --- Etiquettes de genre
            
            if ($this->oLatin->isMasculine)  // --- Masculin
                {
                echo ($this->oLatin->isSingular) ?  '<td class="gnr">Mascul.</td>' : '<td class="gnrx">Mascul.</td>';
                }
            else
                {
                echo '<td class="gnrx">Mascul.</td>';
                }
            
            if ($this->oLatin->isFeminine)   // --- Féminin
                {
                echo ($this->oLatin->isSingular) ?  '<td class="gnr">Femin.</td>' : '<td class="gnrx">Femin.</td>';
                }
            else
                {
                echo '<td class="gnrx">Femin.</td>';
                }
                
            if ($this->oLatin->isNeutral)    // --- Neutre
                {
                echo ($this->oLatin->isSingular) ?  '<td class="gnr">Neutr.</td>' : '<td class="gnrx">Neutr.</td>';
                }
            else
                {
                echo '<td class="gnrx">Neutr.</td>';
                }
                
            echo '</tr>';
              
            // --- Cases de déclinaison
            
            echo '<tr><td>Nom.</td>';
            $this->writeCase(0, $this->oLatin->Decl);
            $this->writeCase(12, $this->oLatin->Decl);      
            $this->writeCase(24, $this->oLatin->Decl);      
            echo '</tr>';
                  
            echo '<tr><td>Voc.</td>';
            $this->writeCase(1, $this->oLatin->Decl);
            $this->writeCase(13, $this->oLatin->Decl);      
            $this->writeCase(25, $this->oLatin->Decl);      
            echo '</tr>';
                
            echo '<tr><td>Acc.</td>';
            $this->writeCase(2, $this->oLatin->Decl);
            $this->writeCase(14, $this->oLatin->Decl);      
            $this->writeCase(26, $this->oLatin->Decl);      
            echo '</tr>';
            
            echo '<tr><td>Gen.</td>';
            $this->writeCase(3, $this->oLatin->Decl);
            $this->writeCase(15, $this->oLatin->Decl);      
            $this->writeCase(27, $this->oLatin->Decl);      
            echo '</tr>';
            
            echo '<tr><td>Dat.</td>';
            $this->writeCase(4, $this->oLatin->Decl);
            $this->writeCase(16, $this->oLatin->Decl);      
            $this->writeCase(28, $this->oLatin->Decl);      
            echo '</tr>'; 

            echo '<tr><td>Abl.</td>';
            $this->writeCase(5, $this->oLatin->Decl);
            $this->writeCase(17, $this->oLatin->Decl);      
            $this->writeCase(29, $this->oLatin->Decl);      
            echo '</tr>'; 
            
            
            
            // ----------------------------------------
            // --- Affichage des cases du Pluriel      
            // ----------------------------------------
            
            echo '<tr><td><b>Plur.</b></td>';
            
            // --- Etiquettes de genre
            
            if ($this->oLatin->isMasculine)  // --- Masculin
                {
                echo ($this->oLatin->isPlural) ?  '<td class="gnr">Mascul.</td>' : '<td class="gnrx">Mascul.</td>';
                }
            else
                {
                echo '<td class="gnrx">Mascul.</td>';
                }
            
            if ($this->oLatin->isFeminine)   // --- Féminin
                {
                echo ($this->oLatin->isPlural) ?  '<td class="gnr">Femin.</td>' : '<td class="gnrx">Femin.</td>';
                }
            else
                {
                echo '<td class="gnrx">Femin.</td>';
                }
                
            if ($this->oLatin->isNeutral)    // --- Neutre
                {
                echo ($this->oLatin->isPlural) ?  '<td class="gnr">Neutr.</td>' : '<td class="gnrx">Neutr.</td>';
                }
            else
                {
                echo '<td class="gnrx">Neutr.</td>';
                }
                
            echo '</tr>';
            
            // --- Cases de déclinaison
            
            echo '<tr><td>Nom.</td>';
            $this->writeCase(6, $this->oLatin->Decl);
            $this->writeCase(18, $this->oLatin->Decl);      
            $this->writeCase(30, $this->oLatin->Decl);      
            echo '</tr>'; 
            
            echo '<tr><td>Voc.</td>';
            $this->writeCase(7, $this->oLatin->Decl);
            $this->writeCase(19, $this->oLatin->Decl);      
            $this->writeCase(31, $this->oLatin->Decl);      
            echo '</tr>'; 
                  
            echo '<tr><td>Acc.</td>';
            $this->writeCase(8, $this->oLatin->Decl);
            $this->writeCase(20, $this->oLatin->Decl);      
            $this->writeCase(32, $this->oLatin->Decl);      
            echo '</tr>'; 
            
            echo '<tr><td>Gen.</td>';
            $this->writeCase(9, $this->oLatin->Decl);
            $this->writeCase(21, $this->oLatin->Decl);      
            $this->writeCase(33, $this->oLatin->Decl);      
            echo '</tr>';     
            
            echo '<tr><td>Dat.</td>';
            $this->writeCase(10, $this->oLatin->Decl);
            $this->writeCase(22, $this->oLatin->Decl);      
            $this->writeCase(34, $this->oLatin->Decl);      
            echo '</tr>'; 
   
            echo '<tr><td>Abl.</td>';
            $this->writeCase(11, $this->oLatin->Decl);
            $this->writeCase(23, $this->oLatin->Decl);      
            $this->writeCase(35, $this->oLatin->Decl);      
            echo '</tr>';                    
            
            echo '</table></td></tr></table>';
            
            // --- Infos sur les cas
            
            echo '<p class="exem"><b>Nominatif</b> : cas du sujet et de l\'attribut du sujet.<br />
            <b>Vocatif</b> : cas de l\'apostrophe, de l\'interpellation. Ce cas est inusité en contexte technique (latin scientifique).<br />
            <b>Accusatif</b> : cas du complément d\'objet direct et de son attribut.<br />
            <b>Génitif</b> : cas du complément du nom.<br />
            <b>Datif</b> : cas du complément d\'objet indirect (<i>à quelqu\'un ou quelque chose</i>) et du complément d\'attribution (<i>à quelqu\'un ou quelque chose</i>).<br />
            <b>Ablatif</b> : cas des compléments circonstanciels (de moyen, de manière, d\'origine) ou d\'état (avec...).
            </p>';
        
            }
        // =============================================================================
        elseif ($this->oLatin->isConjugated)     // === Le mot a été effectivement conjugué   
        // =============================================================================
            {
            
            $this->AfficherVerbe();
        
            }
        // ==============================================================
        else // === Pas de décinaison ni conjugaison effective           
        // ==============================================================
            {
            
            
            // --- Message
            if ($motBrut == '')
                {
                echo '<p class="warn">Sélectionnez ou tapez un mot...</p>';
                }
            elseif ($this->oLatin->isDeclNone)
                {
                echo '<p class="warn">Cette expression n\'est pas déclinable/conjugable.</p>';
                }
            elseif ($this->oLatin->isDeclUnknown)
                {
                echo '<p class="warn">La déclinaison/conjugaison de cette expression n\'est pas affichable.</p>';
                }
            else
                {
                echo '<p class="warn">Impossible d\'afficher une déclinaison/conjugaison.</p>';
                }
        
            }
        
        // =========================================
        // === Lien vers le script                  
        // =========================================
        
        echo '<p>&nbsp;</p><hr /><p class="toscript">Animé par <b>Declinator</b>, greffon de déclinaisons et conjugaisons latines pour <a href="http://linguae.stalikez.info">SqLinguae</a>, visualiseur libre de dictionnaire en ligne.</p><p>&nbsp;</p>';
        
            
        return;
    
        }
    
    
    
    function writeCase($n, $array)
        {
        if ($array[$n]->Rac == '' && $array[$n]->Des == '')
            {
            echo '<td class="greycase">&nbsp;</td>';
            }
        else
            {
            echo '<td class="case">', $array[$n]->Rac, '<span class="des">', $array[$n]->Des, '</span></td>';
            }
            
        }
    
    
    }


?>
